//****************************************************
//*
//*  Sample UserAnalysis Script 
//*  
//****************************************************
#include "TNtupleD.h"
#include <sstream>

TCanvas *cHist;
TH1F *hNCDC = 0;
TH1F *hNVTX = 0;
TH1F *hNGen = 0;
TH1F *hMjet = 0;
TNtupleD *hEvt = 0;
Int_t nev   = 0;

//_________________________________________________________
void UserInitialize()
{
  //  This function is called at the begining of the job or when
  //  "reset hist" action is selected in the gui menu.
  //  This is used to define/reset histograms.

  if( !hNCDC ) delete hNCDC; hNCDC = 0;
  if( !hNVTX ) delete hNVTX; hNVTX = 0;
  if( !hNGen ) delete hNGen; hNGen = 0;
  if( !hMjet ) delete hMjet; hMjet = 0;
  hNCDC=new TH1F("hNCDC","Number of CDC Tracks",100,0,100);
  hNVTX=new TH1F("hNVTX","Number of VTX Hits",100,0,200);
  hNGen=new TH1F("hNGen","Number of Generator Tracks",100,0,200);
  hMjet=new TH1F("hMjet","Jet Mass",400,0,200);
  if( !hEvt ) delete hEvt; hEvt = 0;
  stringstream tupstr;
  tupstr << "ntracks:evis:pt:pl:mqq:ej1:ej2:cosj1:cosj2"
         << ":mj1:mj2:cosz:npj1:npj2:egam:cosgam"
         << ends;
  hEvt = new TNtupleD("hEvt", "", tupstr.str().data());
}

//_________________________________________________________
void UserAnalysis()
{
  nev++;
  cerr << "event " << nev << endl;
  // This function is called when the processing of one event is completed.
  // Any data processing of the event can be performed in this function.
  // 

#if 0
  JSFSpring    *sp =(JSFSpring*)jsf->FindModule("AZSpring");
  JSFSpringBuf *spb=(JSFSpringBuf*)sp->EventBuf();
  Int_t nsps = spb->GetNpartons();
  cerr << " --- Npartons = " << nsps << " -------- " << endl;
  ANL4DVector pcm;
  TClonesArray *partons = spb->GetPartons();
  TIter next(partons);
  JSFSpringParton *parton;
  while ((parton = (JSFSpringParton *)next())) {
    cerr << parton->GetSerial() << " "
         << " PID=" << parton->GetID()
         << " Q="   << parton->GetCharge()
         << " M="   << parton->GetMass()
         << " M="   << parton->GetColorID()
         << " M="   << parton->GetShowerInfo()
	 << " p=("  << parton->GetE() << ","
	            << parton->GetPx() << ","
	            << parton->GetPy() << ","
	            << parton->GetPz() << ")" << endl;
    if (!parton->GetNDaughter()) continue;
    pcm += ANL4DVector(parton->GetPV());
  }
  cerr << " SpringParton : ";
  pcm.DebugPrint();
#endif


  JSFSIMDSTBuf *sdb=(JSFSIMDSTBuf*)simdst->EventBuf();
  //  Accumulate information in the histogram
  hNCDC->Fill((Float_t)sdb->GetNCDCTracks());
  hNVTX->Fill((Float_t)sdb->GetNVTXHits());
  hNGen->Fill((Float_t)sdb->GetNGeneratorParticles());

  /*
  **  If these comments are removed, generator particle information
  **  are printed.
  */
  ANL4DVector qcm;
  ANL4DVector qsum;
  ANL4DVector qphoton;
  TObjArray genparticles;
  genparticles.SetOwner();
#if 1
  //printf(" # Generator Particle is %d\n",sdb->GetNGeneratorParticles());
  TClonesArray *gen=sdb->GetGeneratorParticles();
  for(Int_t i=0;i<sdb->GetNGeneratorParticles();i++){
    JSFGeneratorParticle *g=(JSFGeneratorParticle *)gen->UncheckedAt(i);
    Int_t ndau=g->GetNDaughter();
    if( ndau != 0 ) continue;
    // printf(" ndau=%d\n",ndau);
    qcm += ANL4DVector(g->GetPV());
    //g->ls();
    Int_t pid=g->GetID();
    if( pid == 12 || pid == 14 || pid == 16 ) continue;
    Int_t mom=g->GetMother();
    //cerr << " pid = " << pid << " mom = " << mom << endl;
    if( pid == 22 && mom == -1 ) {
	    qphoton = ANL4DVector(g->GetPV());
	    continue;
    }
    ANL4DVector *qp = new ANL4DVector(g->GetPV());
    genparticles.Add(qp);
    qsum += *qp;
  }
#if 0
  cerr << " GeneratorParticle : ";
  qcm.DebugPrint();
#endif

  Double_t ycut = 0.004;
  ANLJadeEJetFinder jclust(ycut);
  jclust.Initialize(genparticles);
  jclust.FindJets();
  //cerr << " Njets = " << jclust.GetNjets() << endl;
  jclust.ForceNJets(2);
  TObjArray &jets = jclust.GetJets();
  ANLJet &j1 = *(ANLJet *)jets[0]; 
  ANLJet &j2 = *(ANLJet *)jets[1]; 
  Int_t npj1 = j1.GetNparticles();
  Int_t npj2 = j2.GetNparticles();
  Double_t mj1 = j1.GetMass();
  Double_t mj2 = j2.GetMass();
  Double_t ej1 = j1.E();
  Double_t ej2 = j2.E();
  Double_t csj1 = j1.CosTheta();
  Double_t csj2 = j2.CosTheta();

  ANL4DVector z = j1 + j2;

  Int_t ntracks = genparticles.GetEntries() + 1;
  Double_t evis = qsum(0);	// E_vis
  Double_t pt	= qsum.GetPt(); // P_t
  Double_t pl	= qsum(3);	// P_l

  Double_t mqq       = z.GetMass();
  Double_t cosz      = z.CosTheta();
  Double_t ephoton   = qphoton.E();
  Double_t cosphoton = qphoton.CosTheta();

  Double_t data[100];
  data[ 0] = ntracks;
  data[ 1] = evis;
  data[ 2] = pt;
  data[ 3] = pl;
  data[ 4] = mqq;
  data[ 5] = ej1;
  data[ 6] = ej2;
  data[ 7] = csj1;
  data[ 8] = csj2;
  data[ 9] = mj1;
  data[10] = mj2;
  data[11] = cosz;
  data[12] = npj1;
  data[13] = npj2;
  data[14] = ephoton;
  data[15] = cosphoton;

  hEvt->Fill(data);

#if 0
  cerr << " npj1=" << npj1 << " Mj1=" << mj1
       << " npj2=" << npj2 << " Mj2=" << mj2 << endl;
#endif
  Int_t xNp = 3;
  if (npj1 > xNp && npj2 > xNp) {
	  hMjet->Fill(mj1);
	  hMjet->Fill(mj2);
  }
#endif
}

//_________________________________________________________
void DrawHist()
{
  //  This function is called to draw histograms during the interactive 
  //  session.  Thus you can see the accumulation of the histogram
  //  interactively.  

  TDirectory *old=gDirectory;
  if( !cHist ) {
    cHist=new TCanvas("cHist","Canvas 1",100, 100, 800, 800);
    cHist->Divide(2,2);
  } 
  else {
    cHist->cd();
  }
  cHist->cd(1);
  hNCDC->Draw();
  cHist->cd(2);
  hNVTX->Draw();
  cHist->cd(3);
  hNGen->Draw();
  cHist->cd(4);
  hMjet->Draw();
  cHist->Update();

  old->cd();
}

//_________________________________________________________
void UserSetOptions()
{
  // This function is called only once, soon after jsf is started.
  // This function can be used to define parameters which is not 
  // defined in jsf.conf file.

}

//_________________________________________________________
void UserTerminate()
{
  // This function is called at the end of job.
  cerr << "UserTerminate() *****" << endl;

}
