//
// itanl.C
//
// 

#include <iomanip>

void itanl()
{
//----------------------------------
// Reset ROOT
//----------------------------------

  gROOT->Reset();

  gErrorAbortLevel = kWarning + 1; 
  gRandom->SetSeed();
       
//----------------------------------
// Create JSF runtime environment
//----------------------------------

  JSFSteer *jsfp = new JSFSteer();
    
//----------------------------------
// Load user defined classes
//----------------------------------

  gSystem->Load("libS4Kalman.so");  
  gSystem->Load("libS4Geom.so");  
  gSystem->Load("libS4KalTrack.so");  
  gSystem->Load("libS4GENMCTruth.so");
  gSystem->Load("libS4HISTMCTruth.so");
  gSystem->Load("libU4VHitMaker.so");
  gSystem->Load("libU4KalDetector.so");  
  gSystem->Load("libU4TPCKalDetector.so");  
  gSystem->Load("libU4TPCHitMaker.so");
  gSystem->Load("libS4TPCMCTruth.so");
  gSystem->Load("libS4TPCHitMaker.so");
  gSystem->Load("libU4ITKalDetector.so");  
  gSystem->Load("libU4ITHitMaker.so");
  gSystem->Load("libS4ITMCTruth.so");
  gSystem->Load("libS4ITHitMaker.so");
  gSystem->Load("libS4VTXMCTruth.so");
  gSystem->Load("libU4VTrackMaker.so");  
  gSystem->Load("libS4Kern.so");
  gSystem->Load("libU4TPCTrackMaker.so");  
  gSystem->Load("libS4TPCTrackMaker.so");  
  gSystem->Load("libU4ITTrackMaker.so");  
  gSystem->Load("libS4ITTrackMaker.so");  
  gSystem->Load("libU4HYBTrackMaker.so");
  gSystem->Load("libS4HYBTrackMaker.so");
  gSystem->Load("libS4TPCAnalysis.so");
  gSystem->Load("libS4ITAnalysis.so");

//----------------------------------
// Set output file
//----------------------------------

  TFile *foutp = new TFile("jsf.itanl.root","RECREATE");
  TFile &fout  = *foutp;
  jsfp->SetOutput(fout);

//----------------------------------
// Set input file
//----------------------------------
#if 0
  S4VMCTruth::SetInputFileName("hit.dat");
#else
  TFile file("jsfj4.it.root");
  file.ls();
  jsfp->SetInput(file);
#endif
  Int_t nevent=jsfp->Env()->GetValue("JSFSteer.Nevent",999999);  
  Int_t minevt=1;
  Int_t maxevt=minevt+nevent;

//----------------------------------
// Insert modules
//----------------------------------

  JSFJ4BufManager *bufman         = new JSFJ4BufManager();

  S4GENMCTruth    *mctruthgen     = new S4GENMCTruth();
  S4HISTMCTruth   *mctruthhist    = new S4HISTMCTruth();
  S4VTXMCTruth    *vtxmctruth     = new S4VTXMCTruth();
  S4ITMCTruth     *itmctruth      = new S4ITMCTruth();
  S4TPCMCTruth    *tpcmctruth     = new S4TPCMCTruth();
  S4ITHitMaker    *ithitmaker     = new S4ITHitMaker();
  S4TPCHitMaker   *tpchitmaker    = new S4TPCHitMaker();
  S4ITTrackMaker  *ittrackmaker   = new S4ITTrackMaker();
  S4TPCTrackMaker *tpctrackmaker  = new S4TPCTrackMaker();
  S4HYBTrackMaker *hybtrackmaker  = new S4HYBTrackMaker();
  S4TPCAnalysis   *tpcanalysis    = new S4TPCAnalysis();
  S4ITAnalysis    *itanalysis     = new S4ITAnalysis();

  bufman       ->SetMakeBranch(kFALSE);
  tpcmctruth   ->SetMakeBranch(kFALSE);
  itmctruth    ->SetMakeBranch(kFALSE);
  tpchitmaker  ->SetMakeBranch(kFALSE);
  tpctrackmaker->SetMakeBranch(kFALSE);
  ithitmaker   ->SetMakeBranch(kFALSE);
  ittrackmaker ->SetMakeBranch(kFALSE);
  hybtrackmaker->SetMakeBranch(kFALSE);
  tpcanalysis  ->SetMakeBranch(kFALSE);
  itanalysis   ->SetMakeBranch(kFALSE);

//----------------------------------
// set parameters
//----------------------------------
   S4TPCHit::SetSpatialResolution("experimental"); // flat or experimental
   S4TPCHit::SetT0offset(7.);              // nsec
   //S4TPCHitMakerBuf::SwitchOnMergeHit();
   //S4TPCHitMakerBuf::SetMergeRange(1., 5.);

//----------------------------------
// Job initialization
//----------------------------------
 
  jsfp->Initialize();

//----------------------------------
// Define KalDetector
//----------------------------------

  TKalDetCradle    *cradle = new TKalDetCradle();
  U4KalDetector    *det    = new U4KalDetector();
  U4TPCKalDetector *tpcdet = new U4TPCKalDetector("FillLayer");
  U4ITKalDetector  *itdet  = new U4ITKalDetector("FillLayer");
  tpctrackmaker->SetDetector(*tpcdet);
  ittrackmaker ->SetDetector(*itdet);
  hybtrackmaker->SetDetector(*det);

  //itdet->FBOff();
  cradle->Install(*det);
  cradle->Install(*tpcdet);
  cradle->Install(*itdet);
  cradle->Close();
  //cradle->SwitchOffMS();
  //cradle->SwitchOffDEDX();

//----------------------------------
// set subtrackers to hybtrackmaker 
//----------------------------------

  hybtrackmaker->SetMainTrackMakerPtr(tpctrackmaker);
  hybtrackmaker->AddSubTrackMakerPtr(ittrackmaker);

//----------------------------------
// Start event loop
//----------------------------------

  for(Int_t ev=minevt;ev<=maxevt;ev++){
    if (!jsfp->GetEvent(ev)) break; 
    cerr << " ---- Event " << jsfp->GetEventNumber() << " ----" << endl;
    if (!jsfp->Process(ev)) {
      jsfp->FillTree();
      jsfp->Clear();
      break;
    }
    jsfp->FillTree();
    jsfp->Clear();
  }
  jsfp->Terminate();
  fout.Write();
}
