//////////////////////////////////////////////////////////////////
//
//  Geant4 execution in JSF framework
//
//
//$Id: JSFFromLCIO.cxx,v 1.2 2007/03/20 09:36:10 fujiik Exp $
//
//////////////////////////////////////////////////////////////////

#include <iostream>

#include "JSFSteer.h"
#include "JSFModule.h"
#include "JSFFromLCIO.h"


#include "S4FCALExactHit.h"
#include "S4FCALPreHit.h"
#include "S4BCALExactHit.h"
#include "S4BCALPreHit.h"
#include "S4VTXExactHit.h"

#include "S4ITExactHit.h"
#include "S4TPCExactHit.h"
#include "S4TPCPostHit.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4MUDExactHit.h"
#include "S4MUDPreHit.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TMath.h"
#endif


ClassImp(JSFFromLCIO)

//_____________________________________________________________________________
JSFFromLCIO::JSFFromLCIO(const char *name, const char *title)
  : JSFJ4(name,title)
{

}

//_____________________________________________________________________________
JSFFromLCIO::~JSFFromLCIO()
{

}

//_____________________________________________________________________________
Bool_t JSFFromLCIO::Initialize()
{
  if( !IsWritable() ) return kTRUE;

  JSFJ4Buf *buf=(JSFJ4Buf*)EventBuf();
  TObjArray *fcalhits=buf->AddComponent("FCALHits");
  TObjArray *fcalprehits=buf->AddComponent("FCALPreHits");
  TObjArray *vtxhits=buf->AddComponent("VTXHits");
  TObjArray *bcalhits=buf->AddComponent("BCALHits");
  TObjArray *bcalprehits=buf->AddComponent("BCALPreHits");
  TObjArray *ithits=buf->AddComponent("ITHits");
  TObjArray *tpchits=buf->AddComponent("TPCHits");
  TObjArray *tpcposthits=buf->AddComponent("TPCPostHits");
  TObjArray *calhits=buf->AddComponent("CALHits");
  TObjArray *calprehits=buf->AddComponent("CALPreHits");

  TObjArray *mudhits=buf->AddComponent("MUDHits");
  TObjArray *mudprehits=buf->AddComponent("MUDPreHits");

  return kTRUE;
}

//_____________________________________________________________________________
Bool_t JSFFromLCIO::Terminate()
{
  if( !IsWritable() ) return kTRUE;
  return kTRUE;
}

//_____________________________________________________________________________
Bool_t JSFFromLCIO::BeginRun(Int_t irun)
{

  if( !IsWritable() ) return kTRUE;

  return kTRUE;
}

//_____________________________________________________________________________
Bool_t JSFFromLCIO::EndRun()
{

  if( !IsWritable() ) { return kTRUE; }

  return kTRUE;
}



//_____________________________________________________________________________
Bool_t JSFFromLCIO::Process(Int_t nev)
{
//

  JSFJ4Buf *buf=(JSFJ4Buf*)EventBuf();
  buf->Clear();

  // Fill dummy hit data
  TObjArray *fcalhits=buf->GetComponent("FCALHits");
  TObjArray *fcalprehits=buf->GetComponent("FCALPreHits");
  TObjArray *vtxhits=buf->GetComponent("VTXHits");
  TObjArray *bcalhits=buf->GetComponent("BCALHits");
  TObjArray *bcalprehits=buf->GetComponent("BCALPreHits");
  TObjArray *ithits=buf->GetComponent("ITHits");
  TObjArray *tpchits=buf->GetComponent("TPCHits");
  TObjArray *tpcposthits=buf->GetComponent("TPCPostHits");
  TObjArray *calhits=buf->GetComponent("CALHits");
  TObjArray *calprehits=buf->GetComponent("CALPreHits");

  TObjArray *mudhits=buf->GetComponent("MUDHits");
  TObjArray *mudprehits=buf->GetComponent("MUDPreHits");
  for(Int_t i=0;i<5;i++) {
    Double_t r=i*2+1;
    TVector3 prepos(r, 0, 0);
    TVector3 postpos(r, 0, 0);
    TVector3 momentum(3.5, 3.5, 0.0);
    vtxhits->Add(new S4VTXExactHit(nev, i%4, i, 0, i, 311, 1, 
					 prepos, postpos, momentum,
					 3.5*sqrt(2), 1.2, 4.5));
    
    calhits->Add(new S4CALExactHit());

    fcalhits->Add(new S4FCALExactHit());
    fcalprehits->Add(new S4FCALPreHit());
    bcalhits->Add(new S4BCALExactHit());
    bcalprehits->Add(new S4BCALPreHit());
    
    ithits->Add(new S4ITExactHit());
    tpchits->Add(new S4TPCExactHit());
    tpcposthits->Add(new S4TPCPostHit());
    
    calprehits->Add(new S4CALPreHit());
    mudhits->Add(new S4MUDExactHit());
    mudprehits->Add(new S4MUDPreHit());

  }

  
  buf->SetEventID(nev);


  // Output data

  std::cerr << "**********************************************" << std::endl;
  std::cerr << "Number of hit is " << vtxhits->GetEntries() << std::endl;
  std::cerr << "**********************************************" << std::endl;
  TIter next(vtxhits);
  S4VTXExactHit *vtx;
  while( (vtx=(S4VTXExactHit*)next())) {
    vtx->Print();
  }


  Bool_t runstat=kTRUE;
  if ( runstat ) {
    return kTRUE;
  }
  else {
    gJSF->SetReturnCode(JSFSteer::kJSFDoEndRun) ;
    return kTRUE;
  }
}

