///////////////////////////////////////////////////////////////////
//
//  CAL Hit class in ROOT
//
//$Id: S4BCALExactHit.cxx,v 1.4 2006/07/03 07:14:10 miyamoto Exp $
//
//////////////////////////////////////////////////////////////////

#include <iomanip>
#include <iostream>

#ifndef __HasJSFFromLCIO__
#include "J4IRBCALHit.hh"
#endif
#include "S4BCALExactHit.h"
using namespace std;

ClassImp(S4BCALExactHit)

// ------------------------------------------------------------------
S4BCALExactHit::S4BCALExactHit()
              : S4VExactHit(0,                 // hit ID
                            0,                 // track ID
                            0,                 // mother track ID
                            0,                 // PDG code
                            0,                 // charge
                            TVector3(0.,0.,0.),// position
                            0,                 // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            0,                 // energy deposit
                            0),                // address ptr 
                fPreHitID(0),
                fCellID(0),
		//                fIsBarrel(kTRUE),
		//                fIsEM(kTRUE),
		f1stHit(0.0, 0.0, 0.0)
		// fPreHitPtr(0)
{
}

/*
// ------------------------------------------------------------------
S4BCALExactHit::S4BCALExactHit(Int_t     hitID,       // hit ID
                             Int_t     pcode,       // PDG code
                      const TVector3  &pos,         // position [cm]
                             Double_t  tof,         // TOF      [nsec]
                             Double_t  edep,        // energy deposit [GeV]
                             Int_t     preHitID,    // pre hit ID
                             Int_t     cellID,      // cell ID
                             Int_t     isBarrel,    // energy deposit [GeV]
                             Int_t     isEM,        // energy deposit [GeV]
                       const TVector3 &xcell,       // cell position [cm]
                       const TVector3 &xcm,         // E-weighted pos [cm*GeV]
			     Int_t     trackID,
			     Int_t     mtrackID)
              : S4VExactHit(hitID,             // hit ID
                            trackID,           // track ID
                            mtrackID,          // mother track ID
                            pcode,             // PDG code
                            0,                 // charge
                            pos,               // position
                            tof,               // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            edep,              // energy deposit
                            0),                // address ptr 
                fPreHitID(preHitID),
                fCellID(cellID),
                fIsBarrel(isBarrel),
                fIsEM(isEM),
                fCellPosition(xcell),
                fCellXcm(xcm),
                fPreHitPtr(0)
{
}
*/

#ifndef __HasJSFFromLCIO__
// ------------------------------------------------------------------
S4BCALExactHit::S4BCALExactHit(J4IRBCALHit *ahit)
              : S4VExactHit(ahit->GetPreHitID(),        // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            ahit->GetCharge(),          // charge
                         TVector3(ahit->GetXcm().x() // E-weighted hit pos
                                /ahit->GetEnergyDeposit()/cm,
    		                     ahit->GetXcm().y()
                                        /ahit->GetEnergyDeposit()/cm,
    		                     ahit->GetXcm().z()
                                        /ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(ahit->GetMomentum().x(),
				     ahit->GetMomentum().y(),
				     ahit->GetMomentum().z()),   // momentum
                            ahit->GetTotalEnergy(),             // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
		//                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fPreHitID(ahit->GetPreHitID()),     // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
		//                fIsBarrel(ahit->IsBarrel()),            // is Barrel?
		//                fIsEM(ahit->IsEM()),                    // is EM?
		f1stHit(ahit->GetXhit().x()/cm, ahit->GetXhit().y()/cm, 
			ahit->GetXhit().z()/cm)  // 1st hit position
                // fPreHitPtr(0)
{
}
#endif

// ------------------------------------------------------------------
S4BCALExactHit::~S4BCALExactHit()
{
}

// ------------------------------------------------------------------
void S4BCALExactHit::Print(const Option_t *opt) const
{
  std::cout << std::setiosflags(std::ios::fixed);
  std::cout << "S4BCALExactHit ====================";
  std::cout << " CellID=" << fCellID;
  std::cout << " PDGID=" << GetPDGCode();
  //  std::cout << " IsBarrel=" << fIsBarrel;
  //  std::cout << " IsEM=" << fIsEM;
  std::cout << " TOF(nsec)=" << GetTOF();
  std::cout << " EnergyDeposit(GeV)=" << GetEnergyDeposit();
  //  std::cout << std::endl;
  std::cout << " HitPosition(cm)=(" << GetPosition().x();
  std::cout << "," << GetPosition().y();
  std::cout << "," << GetPosition().z();
  std::cout << ") ";
  std::cout << " 1stHitPos.(cm)=(" << f1stHit.x();
  std::cout << "," << f1stHit.y();
  std::cout << "," << f1stHit.z();
  std::cout << ") ";
  std::cout << std::endl;
}

//---------------------------------------------------------------
Int_t S4BCALExactHit::Compare(const TObject *hit) const
{
  return 0;
}

//---------------------------------------------------------------
void S4BCALExactHit::SetSortBy(Char_t *opt)
{
}

//---------------------------------------------------------------
Bool_t S4BCALExactHit::IsSortable() { return kTRUE; }
