//$Id: S4CALAddress.cxx,v 1.23 2007/03/20 09:36:10 fujiik Exp $
//********************************************************************
//* ---------------
//*  S4CALAddress
//* ---------------
//*   Utility for converting cell address and position.
//*
//* (Update Record)
//*   2004/10/16  Allister Sanchez  Original (preliminary?) version.
//********************************************************************
#include "S4CALAddress.h"
#include <TVector3.h>
#include <TSystem.h>
#include "JSFSteer.h"
#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include "S4CALTowerGeom.h"
#include "JSFJ4.h"
#include "JSFJupiter.h"
#include <cmath>
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TMath.h"
#endif

//-----------------------------------------------------
//* Check Initialization
Bool_t   S4CALAddress::fgIsCALInitialized=kFALSE;
Bool_t   S4CALAddress::fgIsCLXInitialized=kFALSE;

// ----------------------------------------------------
//* CAL static data members
Bool_t   S4CALAddress::gfIsTowerData=kFALSE;
Double_t S4CALAddress::fgCALTowerFront=0.;

Bool_t   S4CALAddress::fgIsCAL=kFALSE;
Int_t    S4CALAddress::fgCALNTowers=0;
Int_t    S4CALAddress::fgCALNIsEM=0;
Int_t    S4CALAddress::fgCALNEMMiniCones=0;
Int_t    S4CALAddress::fgCALNEMMiniTowers=0;
Int_t    S4CALAddress::fgCALNEMLayers=0;
Int_t    S4CALAddress::fgCALNEMSubLayers=0;
Int_t    S4CALAddress::fgCALNEMCells=0;
Double_t S4CALAddress::fgCALEMAbsLayer=0.;
Double_t S4CALAddress::fgCALEMActiveLayer=0.;
Double_t S4CALAddress::fgCALEMLayer=0.;

Int_t    S4CALAddress::fgCALNHDMiniCones=0;
Int_t    S4CALAddress::fgCALNHDMiniTowers=0;
Int_t    S4CALAddress::fgCALNHDLayers=0;
Int_t    S4CALAddress::fgCALNHDSubLayers=0;
Int_t    S4CALAddress::fgCALNHDCells=0;
Int_t    S4CALAddress::fgCALNBlocks=0;
Double_t S4CALAddress::fgCALHDAbsLayer=0.;
Double_t S4CALAddress::fgCALHDActiveLayer=0.;
Double_t S4CALAddress::fgCALHDLayer=0.;

std::vector<Int_t>    S4CALAddress::fgCALNPhi;
std::vector<Double_t> S4CALAddress::fgCALRadius;
std::vector<Double_t> S4CALAddress::fgCALTheta;
std::vector<Double_t> S4CALAddress::fgCALDTheta;

// ----------------------------------------------------
//* CLX static data members
Bool_t   S4CALAddress::fgIsCLX              = kFALSE;
Double_t S4CALAddress::fgCLXBarrelInnerR    = 0;
Double_t S4CALAddress::fgCLXEndcapInnerR    = 0;
Double_t S4CALAddress::fgCLXBarrelHalfZ     = 0;
Double_t S4CALAddress::fgCLXGap             = 0;
Double_t S4CALAddress::fgCLXEMActiveThick   = 0;
Double_t S4CALAddress::fgCLXHDActiveThick   = 0;
Double_t S4CALAddress::fgCLXEMAbsThick      = 0;
Double_t S4CALAddress::fgCLXHDAbsThick      = 0;
Double_t S4CALAddress::fgCLXEMFlexThick     = 0;
Double_t S4CALAddress::fgCLXHDFlexThick     = 0;
Double_t S4CALAddress::fgCLXCellSize        = 0;
Double_t S4CALAddress::fgCLXEMLayerThick    = 0; 
Double_t S4CALAddress::fgCLXHDLayerThick    = 0;
Double_t S4CALAddress::fgCLXEMThick         = 0;
Double_t S4CALAddress::fgCLXHDThick         = 0;
Double_t S4CALAddress::fgCLXEMYmax          = 0;
Double_t S4CALAddress::fgCLXHDYmax          = 0;
Double_t S4CALAddress::fgCLXEMHalfZ         = 0;
Double_t S4CALAddress::fgCLXHDHalfZ         = 0;
Double_t S4CALAddress::fgCLXEMHeight        = 0;
Double_t S4CALAddress::fgCLXHDHeight        = 0;

Int_t    S4CALAddress::fgCLXNTraps          = 0;
Int_t    S4CALAddress::fgCLXNBlocks         = 0;
Int_t    S4CALAddress::fgCLXNEMLayers       = 0;
Int_t    S4CALAddress::fgCLXNHDLayers       = 0;
Int_t    S4CALAddress::fgCLXNEMTrapStrips   = 0;
Int_t    S4CALAddress::fgCLXNHDTrapStrips   = 0;
Int_t    S4CALAddress::fgCLXNEMStrips       = 0;
Int_t    S4CALAddress::fgCLXNHDStrips       = 0;
Int_t    S4CALAddress::fgCLXNEMCells        = 0;
Int_t    S4CALAddress::fgCLXNHDCells        = 0;
Int_t    S4CALAddress::fgCLXNEMTotals       = 0;

S4CALAddress::S4CALAddress()
{
}

S4CALAddress::~S4CALAddress()
{
}

// ----------------------------------------------------
// Set CAL Parameters
void S4CALAddress::SetCALParameters()
{
  JSFEnv *env = gJSF->Env();

  fgIsCALInitialized = kTRUE;
  fgIsCAL            = env->GetValue("JSFJ4.HasCAL", 1);
  fgCALNTowers       = env->GetValue("J4CAL.Const.NTowers",150);
  fgCALTowerFront    = env->GetValue("J4CAL.BarrelTower.FrontSize",12.0);
  fgCALNIsEM         = env->GetValue("J4CAL.NumberOfCalTypes",2);
  fgCALNEMMiniCones  = env->GetValue("J4CAL.EM.MiniCone.NClones",3);
  fgCALNEMMiniTowers = env->GetValue("J4CAL.EM.MiniTower.NClones",3);
  fgCALNEMLayers     = env->GetValue("J4CAL.EM.NLayers",38);
  fgCALEMAbsLayer    = env->GetValue("J4CAL.EM.AbsLayerThickness",0.4);
  fgCALEMActiveLayer = env->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  Int_t fgCALNSubLayers  = env->GetValue("J4CAL.NSubLayers",2);
  fgCALEMLayer       = fgCALEMAbsLayer+fgCALEMActiveLayer;
  fgCALNEMSubLayers  = fgCALNSubLayers;
  fgCALNEMCells      = fgCALNEMMiniCones*fgCALNEMMiniTowers*fgCALNEMLayers*fgCALNEMSubLayers;
  fgCALNHDMiniCones  = env->GetValue("J4CAL.HD.MiniCone.NClones",1);
  fgCALNHDMiniTowers = env->GetValue("J4CAL.HD.MiniTower.NClones",1);
  fgCALNHDLayers     = env->GetValue("J4CAL.HD.NLayers",130);
  fgCALHDAbsLayer    = env->GetValue("J4CAL.HD.AbsLayerThickness",0.8);
  fgCALHDActiveLayer = env->GetValue("J4CAL.HD.ActiveLayerThickness",0.2);
  fgCALHDLayer       = fgCALHDAbsLayer+fgCALHDActiveLayer;
  fgCALNHDSubLayers  = fgCALNSubLayers;
  fgCALNHDCells      = fgCALNHDMiniCones*fgCALNHDMiniTowers*fgCALNHDLayers*fgCALNHDSubLayers;
  fgCALNBlocks       = fgCALNEMCells + fgCALNHDCells; 

#if 0
  TString FILE = getenv("JUPITERROOT");
  TString file = "/doc/towergeom.dat";
  FILE += file;
#endif
//  TString FILE = "towergeom.dat";

  TString ftowergeom=env->GetValue("J4CAL.TowerGeometryData",""); 
  if( ftowergeom.IsNull() && ! ftowergeom.Contains("/dev/null")) {
    SetTowerParameters(ftowergeom);
  }
  else {
    SetTowerParameters();
  }

}

// ----------------------------------------------------
//* Set CLX Parameters
void S4CALAddress::SetCLXParameters()
{
  JSFEnv *env = gJSF->Env();

  fgIsCLXInitialized = kTRUE;
  fgIsCLX            = env->GetValue("JSFJ4.HasCLX", 1);
  fgCLXNTraps        = env->GetValue("J4CLX.NTraps",12);
  fgCLXBarrelInnerR  = env->GetValue("J4CLX.Barrel.InnerRadius",210.0);
  fgCLXEndcapInnerR  = env->GetValue("J4CLX.Endcap.InnerRadius",40.0);
  fgCLXBarrelHalfZ   = env->GetValue("J4CLX.Barrel.HalfZ",280.0);
  //  fgCLXGap           = env->GetValue("J4CXL.BarrelEndcapGap",10.0);
  fgCLXGap           = env->GetValue("J4CLX.BarrelEndcapGap",10.0);
  fgCLXNEMLayers     = env->GetValue("J4CLX.EM.NLayers",33);
  fgCLXNHDLayers     = env->GetValue("J4CLX.HD.NLayers",46);
  fgCLXNEMStrips     = env->GetValue("J4CLX.EM.NStrips",3);
  fgCLXNHDStrips     = env->GetValue("J4CLX.HD.NStrips",3);  
  fgCLXEMActiveThick = env->GetValue("J4CLX.EM.ActiveLayerThickness",0.2);
  fgCLXHDActiveThick = env->GetValue("J4CLX.HD.ActiveLayerThickness",0.5);
  fgCLXEMAbsThick    = env->GetValue("J4CLX.EM.AbsLayerThickness",0.3);
  fgCLXHDAbsThick    = env->GetValue("J4CLX.HD.AbsLayerThickness",2.0);
  fgCLXEMFlexThick   = env->GetValue("J4CLX.EM.FlexLayerThickness",0.1);
  fgCLXHDFlexThick   = env->GetValue("J4CLX.HD.FlexLayerThickness",0.1);
  fgCLXCellSize      = env->GetValue("J4CLX.Cell.Size",1.0);
  fgCLXEMLayerThick  = fgCLXEMAbsThick+fgCLXEMActiveThick+fgCLXEMFlexThick;
  fgCLXHDLayerThick  = fgCLXHDAbsThick+fgCLXHDActiveThick+fgCLXHDFlexThick;
  fgCLXEMThick       = (fgCLXEMAbsThick+fgCLXEMActiveThick+fgCLXEMFlexThick)*fgCLXNEMLayers;
  fgCLXHDThick       = (fgCLXHDAbsThick+fgCLXHDActiveThick+fgCLXHDFlexThick)*fgCLXNHDLayers;
  fgCLXEMYmax        = fgCLXBarrelInnerR+fgCLXEMThick;
  fgCLXHDYmax        = fgCLXBarrelInnerR+fgCLXEMThick+fgCLXHDThick;
  fgCLXEMHalfZ       = fgCLXBarrelHalfZ-fgCLXGap;
  //  fgCLXHDHalfZ       = fgCLXBarrelHalfZ+fgCLXEMThick-fgCLXGap;  
  fgCLXHDHalfZ       = fgCLXBarrelHalfZ-fgCLXGap;  
  //  fgCLXEMHeight      = fgCLXEMYmax-fgCLXEndcapInnerR-fgCLXGap;
  fgCLXEMHeight      = fgCLXHDYmax-fgCLXEndcapInnerR;  
  fgCLXHDHeight      = fgCLXHDYmax-fgCLXEndcapInnerR;
  
  //fgCLXNBlocks       = 2 * fgCLXNTraps;
  fgCLXNBlocks       = 3 * fgCLXNTraps;
  fgCLXEMLayerThick  = fgCLXEMAbsThick+fgCLXEMActiveThick+fgCLXEMFlexThick;
  fgCLXHDLayerThick  = fgCLXHDAbsThick+fgCLXHDActiveThick+fgCLXHDFlexThick;
  fgCLXNEMTrapStrips = (Int_t)( 2*fgCLXEMHalfZ / fgCLXCellSize );
  fgCLXNHDTrapStrips = (Int_t)( 2*fgCLXHDHalfZ / fgCLXCellSize );
  //  fgCLXNEMCells      = (Int_t)( 2*( fgCLXEMYmax*TMath::Tan( TMath::Pi()/fgCLXNTraps ) / fgCLXCellSize ));
  fgCLXNEMCells      = (Int_t)( 2*( fgCLXHDYmax*TMath::Tan( TMath::Pi()/fgCLXNTraps ) / fgCLXCellSize ));  
  fgCLXNHDCells      = (Int_t)( 2*( fgCLXHDYmax*TMath::Tan( TMath::Pi()/fgCLXNTraps ) / fgCLXCellSize ));
  fgCLXNEMTotals     = fgCLXNEMCells*fgCLXNEMStrips*fgCLXNEMTrapStrips*fgCLXNEMLayers;
}

// ----------------------------------------------------
void S4CALAddress::SetTowerParameters()
{
// Set parameters from data in ROOT data
  gfIsTowerData = kTRUE;
#ifndef __HasJSFFromLCIO__
  JSFJ4 *jsfj4=(JSFJ4*)gJSF->FindModule("JSFJ4");
#else
  JSFJupiter *jsfj4=(JSFJupiter*)gJSF->FindModule("JSFJupiter");
#endif
  S4CALTowerGeom *geom=jsfj4->GetCALTowerGeom();
  std::cerr << "Load CALTowerParameters from JSFJ4 info in root file." << std::endl;
  for(Int_t i=0;i<geom->GetNCones();i++ ) {
      fgCALRadius.push_back(geom->GetRadius(i));
      fgCALTheta.push_back(geom->GetTheta(i));
      fgCALDTheta.push_back(geom->GetDTheta(i));
      fgCALNPhi.push_back(geom->GetNPhi(i));
  }
}

// ----------------------------------------------------
void S4CALAddress::SetTowerParameters(TString ftowergeom)
{
  gfIsTowerData = kTRUE;
  std::cerr << "S4CALAddress::InputFile name "<< ftowergeom << std::endl;
  std::ifstream input_file;
  input_file.open(ftowergeom);
  
  if (!input_file.is_open()){
    std::cerr <<"S4CALAddress " << ftowergeom << " can not open. " << std::endl;
    std::cerr <<" Please set parameter 'J4CALTowerGeometryData' " << std::endl;
    gfIsTowerData = kFALSE;     
    return;
  }
  
  while(!input_file.eof())
    {
      Int_t np;
      Double_t r, th, dth;
      input_file >> r >> th  >> dth >> np;
      fgCALRadius.push_back(r);
      fgCALTheta.push_back(th);
      fgCALDTheta.push_back(dth);
      fgCALNPhi.push_back(np);
    }
  input_file.close();
}

Double_t S4CALAddress::GetDistanceFrom(const TVAddress *addr)
{
  return 0;
}

Double_t S4CALAddress::GetDistanceFrom(const TVector3 pos)
{
  return 0;
}

//------------------------------------------------------------------
//* CAL ID resolvers
//------------------------------------------------------------------
Int_t S4CALAddress::GetConeID(Int_t cellID)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  return cellID / fgCALNBlocks / fgCALNTowers;
}

Int_t S4CALAddress::GetTowerID(Int_t cellID)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  return ( cellID / fgCALNBlocks ) % fgCALNTowers;
}

Int_t S4CALAddress::GetMiniConeID(Int_t cellID, Bool_t isEM)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  return isEM ?
    ( cellID % fgCALNBlocks ) % fgCALNEMMiniCones :
    (( cellID % fgCALNBlocks ) - fgCALNEMCells ) % fgCALNHDMiniCones;
}

Int_t S4CALAddress::GetMiniTowerID(Int_t cellID, Bool_t isEM)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  return isEM ?
    ( cellID % fgCALNBlocks / fgCALNEMMiniCones )   % fgCALNEMMiniTowers :
    ((( cellID % fgCALNBlocks ) - fgCALNEMCells ) / fgCALNHDMiniCones ) % fgCALNHDMiniTowers;
}

Int_t S4CALAddress::GetCALLayerID(Int_t cellID, Bool_t isEM)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  return isEM ?
    (( cellID % fgCALNBlocks ) / fgCALNEMMiniCones / fgCALNEMMiniTowers ) % fgCALNEMLayers :
    ((( cellID % fgCALNBlocks ) - fgCALNEMCells ) / fgCALNHDMiniCones / fgCALNHDMiniTowers ) % fgCALNHDLayers;
}


Int_t S4CALAddress::GetSubLayerID(Int_t cellID, Bool_t isEM)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  return isEM ?
    ((cellID%fgCALNBlocks)/fgCALNEMMiniCones/fgCALNEMMiniTowers/fgCALNEMLayers)%fgCALNEMSubLayers :
    (((cellID%fgCALNBlocks)-fgCALNEMCells)/fgCALNHDMiniCones/fgCALNHDMiniTowers/fgCALNHDLayers)%fgCALNHDSubLayers ;
}

//-----------------------------------------------------------------
//* LayerID switched by fgIsCLX
Int_t S4CALAddress::GetLayerID( Int_t cellID, Bool_t isEM )
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? GetCLXLayerID( cellID, isEM ) : GetCALLayerID( cellID, isEM );
}

// -----------------------------------------------------------------
//* Cell ID converter for CAL
Int_t S4CALAddress::GetCellID( Int_t coneID, Int_t towerID,
			       Int_t miniConeID, Int_t miniTowerID,
			       Int_t layerID, Int_t subLayerID,
			       Bool_t isEM)
{
  if ( !fgIsCALInitialized ) { S4CALAddress::SetCALParameters(); }
  Int_t blockID = isEM ?
    ((( subLayerID*fgCALNEMLayers + layerID )*fgCALNEMMiniTowers + miniTowerID )*fgCALNEMMiniCones + miniConeID) :
    fgCALNEMCells + ((( subLayerID*fgCALNHDLayers + layerID )*fgCALNHDMiniTowers + miniTowerID )*fgCALNHDMiniCones + miniConeID);
  return ( coneID * fgCALNTowers + towerID ) * fgCALNBlocks + blockID;	
}

// -----------------------------------------------------------------
//* Cell ID converter for CLX 
Int_t S4CALAddress::GetCellID( Bool_t isEM, Int_t  blockID,
			       Int_t  layerID, Int_t trapStripID,
			       Int_t  stripID, Int_t  cellID   )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  return ( isEM )
    ? ((( cellID*fgCLXNEMStrips+stripID )*fgCLXNEMTrapStrips + trapStripID)*fgCLXNEMLayers+layerID)*fgCLXNBlocks+blockID
    //    : ( fgCLXNEMTotals + (( cellID*fgCLXNHDStrips*stripID )*fgCLXNHDTrapStrips+trapStripID)*fgCLXNHDLayers+layerID )*fgCLXNBlocks + blockID;
    : ( fgCLXNEMTotals + (( cellID*fgCLXNHDStrips+stripID )*fgCLXNHDTrapStrips+trapStripID)*fgCLXNHDLayers+layerID )*fgCLXNBlocks + blockID;  
}

//-------------------------------------------------------------------
//* IsEndcap
Int_t S4CALAddress::IsEndcap( Int_t cellID, Bool_t /*isBarrel*/ )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t blockID  = GetCLXBlockID( cellID );
  Int_t isEndcap = 0; 
  if ( blockID < fgCLXNTraps )                               isEndcap =  0; // Barrel
  if ( blockID >=   fgCLXNTraps && blockID < 2*fgCLXNTraps ) isEndcap =  1; // Endcap+Z
  if ( blockID >= 2*fgCLXNTraps && blockID < 3*fgCLXNTraps ) isEndcap = -1; // Endcap+Z
  
  //return ( isBarrel == 1 ) ? 0
  //  : ( S4CALAddress::GetCLXBlockID( cellID ) < fgCLXNTraps ) ? 1 : -1;
  return isEndcap;
}

//-------------------------------------------------------------------
//* IsBarrel
Int_t S4CALAddress::IsBarrel( Int_t cellID )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  return ( IsEndcap( cellID ) == 0 ) ? kTRUE : kFALSE;
}

//-------------------------------------------------------------------
//* Check this cell is boundary or not.
// stripID==1 ==> Not boundary, stripID==0 or 2 ==> Boundary
Bool_t S4CALAddress::IsBoundary( Int_t globalID, Bool_t isEM )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t stripID = GetCLXStripID( globalID, isEM );
  return ( stripID == 1 ) ? kFALSE : kTRUE;
}

//-------------------------------------------------------------------
//* X negative position at local axis system.
Bool_t S4CALAddress::IsXNegative( Int_t globalID, Bool_t isEM, Bool_t /*isBarrel*/ )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t cellID      = GetCLXCellID( globalID, isEM );
  Int_t stripID     = GetCLXStripID( globalID, isEM );
  Bool_t isBarrel   = IsBarrel( globalID );

  //* Switch LayerID/trapStripID by barrel/Endcap
  Int_t trapLayerID = ( isBarrel ) ? GetCLXLayerID( globalID, isEM ) : GetCLXTrapStripID( globalID, isEM );

  Int_t nCellsHalf  = (Int_t)( 0.5*GetNCells( trapLayerID, isEM, isBarrel ) );

  return ( (!IsBoundary( globalID, isEM )&&(cellID<nCellsHalf)) ||
	   (IsBoundary( globalID, isEM )&&stripID==0 ) ) ? kTRUE : kFALSE;
}

//------------------------------------------------------------------
//* CLX ID resolvers
//------------------------------------------------------------------
Int_t S4CALAddress::GetCLXBlockID( Int_t cellID )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  return cellID%fgCLXNBlocks;
}

Int_t S4CALAddress::GetCLXLayerID( Int_t cellID, Bool_t isEM )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t remainID = cellID/fgCLXNBlocks;
  return ( isEM ) ? remainID % fgCLXNEMLayers
                  : ( remainID -fgCLXNEMTotals )%fgCLXNHDLayers;
}

Int_t S4CALAddress::GetCLXTrapStripID( Int_t cellID, Bool_t isEM )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t remainID = cellID/fgCLXNBlocks;
  return ( isEM ) ? (remainID/fgCLXNEMLayers)%fgCLXNEMTrapStrips
                  : (( remainID -fgCLXNEMTotals )/fgCLXNHDLayers)%fgCLXNHDTrapStrips;
}

Int_t S4CALAddress::GetCLXStripID( Int_t cellID, Bool_t isEM )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t remainID = cellID/fgCLXNBlocks;
  return ( isEM ) ? (remainID/fgCLXNEMLayers/fgCLXNEMTrapStrips)%fgCLXNEMStrips
                  : (( remainID -fgCLXNEMTotals )/fgCLXNHDLayers/fgCLXNHDTrapStrips)%fgCLXNHDStrips;
}

Int_t S4CALAddress::GetCLXCellID( Int_t cellID, Bool_t isEM )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  Int_t remainID = cellID/fgCLXNBlocks;
  return ( isEM ) ? remainID/fgCLXNEMLayers/fgCLXNEMTrapStrips/fgCLXNEMStrips
                  : ( remainID -fgCLXNEMTotals )/fgCLXNHDLayers/fgCLXNHDTrapStrips/fgCLXNHDStrips;
}

//-------------------------------------------------------------------
//* Number of cells at specified trapStripID or layerID position
// barrel case : layerID, endcap case : trapStripID
Int_t S4CALAddress::GetNCells( Int_t trapLayerID, Bool_t isEM, Bool_t isBarrel )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  Double_t EMLayerThick = fgCLXEMAbsThick+fgCLXEMActiveThick+fgCLXEMFlexThick;
  Double_t HDLayerThick = fgCLXHDAbsThick+fgCLXHDActiveThick+fgCLXHDFlexThick;
    
  Double_t layerThick  = (isEM) ? EMLayerThick : HDLayerThick;
  Double_t EMThick     = EMLayerThick*fgCLXNEMLayers;

  //* Position of activeLayer front surface
  Double_t barrelR     = (isEM) ? fgCLXBarrelInnerR+trapLayerID*layerThick+fgCLXEMAbsThick
                                : fgCLXBarrelInnerR+EMThick+trapLayerID*layerThick+fgCLXHDAbsThick;
  Double_t endcapR     = fgCLXEndcapInnerR+trapLayerID*fgCLXCellSize;

  //* Half X length fixed with cell size
  Double_t halfX       = (isBarrel) ? barrelR*TMath::Tan( TMath::Pi()/fgCLXNTraps )
                                    : endcapR*TMath::Tan( TMath::Pi()/fgCLXNTraps );

  //* Number of cells, always even.
  Int_t    nCells      = 2*(Int_t)(halfX/fgCLXCellSize);

  return nCells;
}

//-------------------------------------------------------------------
//* Local cell ID to search neighbor cells
Int_t S4CALAddress::GetLocalCellID( Int_t globalID, Bool_t isEM, Bool_t isBarrel )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  Int_t cellID       = GetCLXCellID( globalID, isEM );
  Int_t stripLayerID = (isBarrel) ? GetCLXLayerID( globalID, isEM ) : GetCLXTrapStripID( globalID, isEM );
  Int_t nCellsHalf   = (Int_t)(0.5*S4CALAddress::GetNCells( stripLayerID, isEM, isBarrel ));

  //  Int_t localCellID  = ( cellID < nCellsHalf ) ? nCellsHalf-(cellID+1) : cellID-nCellsHalf;
  Int_t localCellID  = ( cellID < nCellsHalf ) ? nCellsHalf-(cellID+1) : cellID-nCellsHalf;

  //* Check cellID is over nCells or negative value
  if ( cellID >= 2*nCellsHalf || cellID < 0 ) {

    std::cerr << "*** Error! CellID=" << cellID << " " 
              << "over the GetNCells()=" <<  2*nCellsHalf << " ! "
	      << std::endl;
  }
  return localCellID;
}

//-------------------------------------------------------------------
//* Front cellID generate from specified cellID
Int_t S4CALAddress::GetFrontCellID( Int_t localID, Int_t stripLayerID,
				    Bool_t isEM, Bool_t isBarrel, Bool_t isXNegative )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  Int_t nCellsHalf   = (Int_t)(0.5*S4CALAddress::GetNCells( stripLayerID-1, isEM, isBarrel ));
  Int_t frontCellID = (isXNegative) ? nCellsHalf-(localID+1) : localID-nCellsHalf;
  
  return frontCellID;
}

//-------------------------------------------------------------------
//* Front cellID generate from specified cellID
Int_t S4CALAddress::GetBackCellID( Int_t localID, Int_t stripLayerID,
				   Bool_t isEM, Bool_t isBarrel, Bool_t isXNegative )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  Int_t nCellsHalf   = (Int_t)(0.5*S4CALAddress::GetNCells( stripLayerID+1, isEM, isBarrel ));
  Int_t frontCellID = (isXNegative) ? nCellsHalf-(localID+1) : localID-nCellsHalf;
  
  return frontCellID;
}  

//-------------------------------------------------------------------
//* Get cellID align cells along the Y axis from local cellID
Int_t S4CALAddress::GetYAlignCellID( Int_t localID, Int_t stripLayerID,
				     Bool_t isEM, Bool_t isBarrel, Bool_t isXNegative )
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  Int_t nCellsHalf   = (Int_t)(0.5*S4CALAddress::GetNCells( stripLayerID, isEM, isBarrel ));
  Int_t yAlignCellID = (isXNegative) ? nCellsHalf-(localID+1) : localID-nCellsHalf;
  
  return yAlignCellID;
}

// -----------------------------------------------------------------
//* CAL/CLX compatible cell ID to Cell position converter
TVector3 S4CALAddress::GetAddressPosition( Int_t globalID, Bool_t isEM, Bool_t /*isBarrel*/ )
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  Double_t px = 0;
  Double_t py = 0;
  Double_t pz = 0;

  TVector3 position( 0., 0., 0. );

  //-------------------------
  //* CAL geometry case
  //-------------------------
  if ( fgIsCAL ) {
    if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();

    Int_t coneID      = S4CALAddress::GetConeID( globalID );
    Int_t towerID     = S4CALAddress::GetTowerID( globalID );
    Int_t miniconeID  = S4CALAddress::GetMiniConeID( globalID, isEM );
    Int_t minitowerID = S4CALAddress::GetMiniTowerID( globalID, isEM );
    //Int_t layerID     = S4CALAddress::GetLayerID( globalID, isEM );
    Int_t layerID     = S4CALAddress::GetCALLayerID( globalID, isEM );
        
    Double_t dPhi = 2 * TMath::Pi() / fgCALNPhi[coneID];
    Double_t startTheta = fgCALTheta[coneID] - 0.5 * fgCALDTheta[coneID];
    Double_t startPhi = dPhi * (towerID - 0.5);
    Double_t cellRho    = isEM ? fgCALRadius[coneID]+fgCALEMLayer*layerID+fgCALEMAbsLayer
                               : fgCALRadius[coneID]+fgCALEMLayer*fgCALNEMLayers+fgCALHDLayer*layerID+fgCALHDAbsLayer ;
    Double_t dCellTheta = isEM ? fgCALDTheta[coneID]/fgCALNEMMiniCones : fgCALDTheta[coneID]/fgCALNHDMiniCones ;
    Double_t dCellPhi   = isEM ? dPhi/fgCALNEMMiniCones : dPhi/fgCALNHDMiniCones ;
    Double_t cellTheta  = startTheta + (miniconeID+0.5)*dCellTheta;
    Double_t cellPhi    = startPhi   + (minitowerID+0.5)*dCellPhi;

    //------------------------------------
    //* Generate Position of x,y,z
    //------------------------------------
    px = cellRho * std::sin(cellTheta)*std::cos(cellPhi);
    py = cellRho * std::sin(cellTheta)*std::sin(cellPhi);
    pz = cellRho * std::cos(cellTheta);
  }

  //-----------------------------
  //* CLX Geometry case
  //-----------------------------
  if ( fgIsCLX ) {
    
    if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

    Int_t cellID       = GetCLXCellID( globalID, isEM );
    //    Int_t stripID      = GetCLXStripID( globalID, isEM );
    Int_t trapStripID  = GetCLXTrapStripID( globalID, isEM );  
    Int_t layerID      = GetCLXLayerID( globalID, isEM );
    Int_t blockID      = GetCLXBlockID( globalID );
    
    Bool_t isBarrel    = IsBarrel( globalID );
    Bool_t isBoundary  = IsBoundary( globalID, isEM );
    Bool_t isXNegative = IsXNegative( globalID, isEM, isBarrel );
    //Int_t  isEndcap    = IsEndcap( globalID, isBarrel );
    Int_t  isEndcap    = IsEndcap( globalID );

    Double_t EMLayerThick = fgCLXEMAbsThick+fgCLXEMActiveThick+fgCLXEMFlexThick;
    Double_t HDLayerThick = fgCLXHDAbsThick+fgCLXHDActiveThick+fgCLXHDFlexThick;
    Double_t layerThick  = (isEM) ? EMLayerThick       : HDLayerThick;
    Double_t activeThick = (isEM) ? fgCLXEMActiveThick : fgCLXHDActiveThick;
    Double_t absThick    = (isEM) ? fgCLXEMAbsThick    : fgCLXHDAbsThick;  
    Double_t EMThick     = EMLayerThick*fgCLXNEMLayers;
    //Double_t HDThick     = HDLayerThick*fgCLXNHDLayers;
    
    //* HalfLength of barrel/endcap and fixed length with cell size
    //Double_t barrelHalfL = (isEM) ? fgCLXBarrelHalfZ-fgCLXGap : fgCLXBarrelHalfZ+EMThick-fgCLXGap;
    Double_t barrelHalfL = fgCLXBarrelHalfZ-fgCLXGap;
    Double_t endcapHalfL = (isEM) ? fgCLXBarrelHalfZ : fgCLXBarrelHalfZ+EMThick;
    Double_t barrelFixedHalfL = fgCLXCellSize*(Int_t)( barrelHalfL/fgCLXCellSize );
  
    //* Position of activeLayer front surface at barrel/endcap
    Double_t barrelR = (isEM) ? fgCLXBarrelInnerR+layerID*EMLayerThick+fgCLXEMAbsThick
                              : fgCLXBarrelInnerR+EMThick+layerID*HDLayerThick+fgCLXHDAbsThick;
    Double_t endcapR = fgCLXEndcapInnerR+trapStripID*fgCLXCellSize;
    
    //* Half length of X/Z
    Double_t halfX   = (isBarrel) ? barrelR*std::tan( M_PI/fgCLXNTraps )
                                  : endcapR*std::tan( M_PI/fgCLXNTraps );
    Double_t halfZ   = (isBarrel) ? -barrelFixedHalfL+(trapStripID+0.5)*fgCLXCellSize
                                  : endcapHalfL+layerID*layerThick+absThick+0.5*activeThick;
  
    //* Half number of cells and boundary cell half length
    Int_t    nCellsHalf = (Int_t)( halfX/fgCLXCellSize );
    Double_t boundHalfX = 0.5*(halfX-fgCLXCellSize*nCellsHalf); // Boundary cells halfX

    //------------------------------------
    //* Generate Position of x,y,z
    //------------------------------------
    px = (isBoundary) ? fgCLXCellSize*nCellsHalf+boundHalfX
                      : -fgCLXCellSize*nCellsHalf+(cellID+0.5)*fgCLXCellSize;
    py = (isBarrel)   ? barrelR+0.5*activeThick
                      : endcapR+0.5*fgCLXCellSize;
    pz = (isBarrel)   ? halfZ
                      : halfZ*isEndcap;
  
    //*  isEndcap is negative (-Z) case, x axis was rotated.
    if ( isEndcap == -1 ) px *= -1;
    if ( isBoundary && isXNegative ) px *= -1.;

    //* Position rotated to -i*dphi direction
    position = TVector3( px, py, pz );
    if ( fgIsCLX ) position.RotateZ( -blockID*2.*M_PI/fgCLXNTraps );
  }
  
  return position;
}

// -----------------------------------------------------------------
//* List of Neighbor Cell IDs
std::vector<Int_t> S4CALAddress::GetListOfNeighbors( Int_t globalID, Bool_t isEM, Bool_t isBarrel, Int_t cells )
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  //* Vector to save neighbor cell IDs
  std::vector<Int_t> neighborVector;

  //---------------------------------
  //* Start converting CAL IDs
  //---------------------------------
  if ( fgIsCAL ) {
    // adjacent cells = 4 + front and back cells = 2
    Int_t coneid      = S4CALAddress::GetConeID(globalID);
    Int_t towerid     = S4CALAddress::GetTowerID(globalID);
    Int_t miniconeid  = S4CALAddress::GetMiniConeID(globalID, isEM);
    Int_t minitowerid = S4CALAddress::GetMiniTowerID(globalID, isEM);
    //Int_t layerid     = S4CALAddress::GetLayerID(globalID, isEM);
    Int_t layerid     = S4CALAddress::GetCALLayerID(globalID, isEM);
    Int_t sublayerid  = S4CALAddress::GetSubLayerID(globalID, isEM);

    Int_t iloop = 0, jloop = 0, kloop = 0;
    if ( isEM ) {
      iloop = fgCALNEMMiniCones*coneid+miniconeid;
      jloop = fgCALNEMMiniTowers*towerid+minitowerid;
    } else {
      iloop = fgCALNHDMiniCones*coneid+miniconeid;
      jloop = fgCALNHDMiniTowers*towerid+minitowerid;
    }
    kloop = layerid;

    for ( Int_t i = iloop-1; i <= iloop+1; i++ ){
      Int_t f1 = TMath::Abs( TMath::Abs( i-iloop )-1 );
      for ( Int_t j = jloop-f1; j <= jloop+f1; j++ ) {
	Int_t f2 = TMath::Abs( TMath::Abs( j-jloop )-1 );
	for ( Int_t k = kloop-f1*f2; k <= kloop+f1*f2; k++ ) {

	  if ( i == iloop && j == jloop && k == kloop ) continue;
	  Int_t neighborcell = 0;

	  if ( isEM ) {
	    neighborcell = S4CALAddress::GetCellID( i/fgCALNEMMiniCones, j/fgCALNEMMiniTowers,
						    i%fgCALNEMMiniCones, j%fgCALNEMMiniTowers,
						    k, sublayerid, isEM );
	  } else {
	    neighborcell = S4CALAddress::GetCellID( i/fgCALNHDMiniCones, j/fgCALNHDMiniTowers,
						    i%fgCALNHDMiniCones, j%fgCALNHDMiniTowers,
						    k, sublayerid, isEM );
	  }
	  // save neighbor cells to the vector
	  neighborVector.push_back( neighborcell );
	}
      }
    }
  }

  //---------------------------------
  //* Start converting CLX IDs
  //---------------------------------
  if ( fgIsCLX ) {

    Int_t cellID       = GetCLXCellID( globalID, isEM );
    Int_t stripID      = GetCLXStripID( globalID, isEM );
    Int_t trapStripID  = GetCLXTrapStripID( globalID, isEM );  
    Int_t layerID      = GetCLXLayerID( globalID, isEM );
    Int_t blockID      = GetCLXBlockID( globalID );
    
    Bool_t isBoundary  = IsBoundary( globalID, isEM );
    Bool_t isXNegative = IsXNegative( globalID, isEM, isBarrel );

    Int_t  trapLayerID = (isBarrel) ? layerID : trapStripID;
    Int_t  nCells      = GetNCells( trapLayerID, isEM, isBarrel );
    //    Int_t  isEndcap    = IsEndcap( globalID, isBarrel );

    //------------------------
    //* Find neighbor cells
    //------------------------
    if ( isBoundary ) { // Boundary cell case ( IsBoundary() = (stripID == 0 or 2) )

      Int_t neighborCell = 0;
      
      //* Front layer cell ID
      neighborCell = GetCellID( isEM, blockID, layerID-1, trapStripID, stripID, cellID );
      neighborVector.push_back( neighborCell );
      //* Backside layer cell ID
      neighborCell = GetCellID( isEM, blockID, layerID+1, trapStripID, stripID, cellID );
      neighborVector.push_back( neighborCell );
      //* Z/Y adjacent cell IDs
      neighborCell = GetCellID( isEM, blockID, layerID, trapStripID-1, stripID, cellID );
      neighborVector.push_back( neighborCell );
      neighborCell = GetCellID( isEM, blockID, layerID, trapStripID+1, stripID, cellID );
      neighborVector.push_back( neighborCell );
      //* X adjacent cell IDs
      neighborCell = (isXNegative) ? GetCellID( isEM, blockID-1, layerID, trapStripID, 2, cellID )
                                   : GetCellID( isEM, blockID+1, layerID, trapStripID, 0, cellID );
      neighborVector.push_back( neighborCell );      
      neighborCell = (isXNegative) ? GetCellID( isEM, blockID, layerID, trapStripID, 1, 0 )
                                   : GetCellID( isEM, blockID, layerID, trapStripID, 1, nCells-1 );
      neighborVector.push_back( neighborCell );      
      
    } else { // center part case ( !IsBoundar() = ( stripID == 1 ) )

      Int_t  localCellID = GetLocalCellID( globalID, isEM, isBarrel );
      Int_t  frontCellID = GetFrontCellID( localCellID, trapLayerID, isEM, isBarrel, isXNegative );
      Int_t  backCellID  = GetBackCellID( localCellID, trapLayerID, isEM, isBarrel, isXNegative );
      
      //* Case adjacent tile is boundary cell
      if ( cellID == 0 || cellID == nCells-1 ) {

	Int_t neighborCell = 0;
	
	//* Front layer cell ID
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID-1, trapStripID, stripID, frontCellID )
                                  : GetCellID( isEM, blockID, layerID-1, trapStripID, stripID, cellID );
	neighborVector.push_back( neighborCell );
	
	//* Backside layer cell ID
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID+1, trapStripID, stripID, backCellID )
                                  : GetCellID( isEM, blockID, layerID+1, trapStripID, stripID, cellID );
	neighborVector.push_back( neighborCell );
	
	//* Z/Y adjacent cell IDs
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID, trapStripID-1, stripID, cellID )
                                  : GetCellID( isEM, blockID, layerID, trapStripID-1, stripID, frontCellID );
	neighborVector.push_back( neighborCell );
	
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID, trapStripID+1, stripID, cellID )
                                  : GetCellID( isEM, blockID, layerID, trapStripID+1, stripID, backCellID );
	neighborVector.push_back( neighborCell );
	
	//* X adjacent cell IDs
	neighborCell = (cellID==0) ? GetCellID( isEM, blockID, layerID, trapStripID, 0, 0 )
                                   : GetCellID( isEM, blockID, layerID, trapStripID, 2, 0 );
	neighborVector.push_back( neighborCell );
	
        neighborCell = (cellID==0) ? GetCellID( isEM, blockID, layerID, trapStripID, 1, cellID+1 )
                                   : GetCellID( isEM, blockID, layerID, trapStripID, 1, nCells-1 );
	neighborVector.push_back( neighborCell );	
	
      } else { // Nomal cell case

	Int_t neighborCell = 0;
	
	//* Front layer cell ID
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID-1, trapStripID, stripID, frontCellID )
	                          : GetCellID( isEM, blockID, layerID-1, trapStripID, stripID, cellID );
	neighborVector.push_back( neighborCell );
	
	//* Backside layer cell ID	
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID+1, trapStripID, stripID, backCellID )
                                  : GetCellID( isEM, blockID, layerID+1, trapStripID, stripID, cellID );
	neighborVector.push_back( neighborCell );
	
	//* Z/Y adjacent cell IDs
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID, trapStripID-1, stripID, cellID )
                                : GetCellID( isEM, blockID, layerID, trapStripID-1, stripID, frontCellID );
	neighborVector.push_back( neighborCell );
	
	neighborCell = (isBarrel) ? GetCellID( isEM, blockID, layerID, trapStripID+1, stripID, cellID )
	                        : GetCellID( isEM, blockID, layerID, trapStripID+1, stripID, backCellID );
	neighborVector.push_back( neighborCell );
	
	//* X adjacent cell IDs
	neighborCell = GetCellID( isEM, blockID, layerID, trapStripID, stripID, cellID-1 );
	neighborVector.push_back( neighborCell );
	neighborCell = GetCellID( isEM, blockID, layerID, trapStripID, stripID, cellID+1 );
	neighborVector.push_back( neighborCell );
	
      }
    }

  }
  
  return neighborVector;
}

// -----------------------------------------------------------------
//* Detector Parameters
Double_t S4CALAddress::GetBarrelInnerR()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXBarrelInnerR
                     : gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
}

Double_t S4CALAddress::GetEndcapInnerR()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXEndcapInnerR
                     : gJSF->Env()->GetValue("J4CAL.Endcap.InnerRadius",39.9);
}

Double_t S4CALAddress::GetBarrelHalfZ()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXBarrelHalfZ
                     : gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
}

Double_t S4CALAddress::GetEMYMax()
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  return fgCLXEMYmax;
}

Double_t S4CALAddress::GetHDYMax()
{
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();
  return fgCLXHDYmax;
}

Double_t S4CALAddress::GetGap()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXGap : 0.;
}

Double_t S4CALAddress::GetEMLayerThickness()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXEMLayerThick : fgCALEMLayer;
}

Double_t S4CALAddress::GetHDLayerThickness()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXHDLayerThick : fgCALHDLayer;
}

Double_t S4CALAddress::GetEMActiveLayerThickness()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXEMActiveThick : fgCALEMActiveLayer;
}

Double_t S4CALAddress::GetHDActiveLayerThickness()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXHDActiveThick : fgCALHDActiveLayer;
}

Double_t S4CALAddress::GetEMAbsLayerThickness()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXEMAbsThick : fgCALEMAbsLayer;  
}

Double_t S4CALAddress::GetHDAbsLayerThickness()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXHDAbsThick : fgCALHDAbsLayer;
}

Int_t    S4CALAddress::GetEMNLayers()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXNEMLayers : fgCALNEMLayers;
}

Int_t    S4CALAddress::GetHDNLayers()
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return ( fgIsCLX ) ? fgCLXNHDLayers : fgCALNHDLayers;
}

Int_t S4CALAddress::GetNMiniCones(Bool_t isEM)
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return isEM ? fgCALNEMMiniCones : fgCALNHDMiniCones;
}

Int_t S4CALAddress::GetNMiniTowers(Bool_t isEM)
{
  if ( !fgIsCALInitialized ) S4CALAddress::SetCALParameters();
  if ( !fgIsCLXInitialized ) S4CALAddress::SetCLXParameters();

  return isEM ? fgCALNEMMiniTowers : fgCALNHDMiniTowers;
}

//-----------------------------------------------
//* Print out parameters
void S4CALAddress::Print( Int_t cellID, Bool_t isEM )
{
  std::cerr << "*** "
            << "blockID=" << S4CALAddress::GetCLXBlockID( cellID ) << " "
            << "layerID=" << S4CALAddress::GetCLXLayerID( cellID, isEM ) << " "
            << "trapStripID=" << S4CALAddress::GetCLXTrapStripID( cellID, isEM ) << " "
            << "stripID=" << S4CALAddress::GetCLXStripID( cellID, isEM ) << " "
            << "cellID="  << S4CALAddress::GetCLXCellID( cellID, isEM ) << " "
            << "isBarrel=" << S4CALAddress::IsBarrel( cellID ) << " "
            << std::endl;
}

//-----------------------------------------------
//* IDs vector
//  0=blockID, 1=layerID, 2=trapStripID, 3=stripID, 4=cellID
//  5=isBarrel, 6=isEndcap
std::vector<Int_t> S4CALAddress::GetCLXIDVector( Int_t cellID, Bool_t isEM ) 
{
   std::vector<Int_t> idVector;
   idVector.push_back( S4CALAddress::GetCLXBlockID( cellID ) );
   idVector.push_back( S4CALAddress::GetCLXLayerID( cellID, isEM ) );
   idVector.push_back( S4CALAddress::GetCLXTrapStripID( cellID, isEM ) );
   idVector.push_back( S4CALAddress::GetCLXStripID( cellID, isEM ) );
   idVector.push_back( S4CALAddress::GetCLXCellID( cellID, isEM ) );
   idVector.push_back( S4CALAddress::IsBarrel( cellID ) );
   idVector.push_back( S4CALAddress::IsEndcap( cellID ) );

   return idVector;
}
