#ifndef __S4CALPostHit__
#define __S4CALPostHit__

//////////////////////////////////////////////////////////////////////
//
//  JSFJ4 CAL PreHit Data
//
//  2004/10/16  Allister Sanchez   Original version.
//
//  $Id: S4CALPostHit.h,v 1.1 2005/02/27 12:57:05 ono Exp $
/////////////////////////////////////////////////////////////////////

#include "S4VExactHit.h"
#include "S4CALPreHit.h"
#include <TVector3.h>
#include <TString.h>

class J4CALPostHit;

// ***************************************************************
class S4CALPostHit : public S4VExactHit
{
 public:
  S4CALPostHit();
  S4CALPostHit(Int_t      postHitID,      // posthit ID
	       Int_t      preHitID,       // prehit ID
	       Int_t      trackID,        // track ID 
	       Int_t      motherTrackID,  // mother track ID
	       Int_t      pcode,          // PDG code
	       Double_t   charge,         // charge
	       TVector3   pos,            // Injection position [cm]
	       Double_t   tof,            // TOF      [nsec]
	       TVector3   mom,            // momentum [GeV] 
	       Double_t   etot,           // energy
	       Double_t   edep,           // energy deposit [GeV]      
	       TVAddress *p = 0);         // address ptr
  S4CALPostHit(J4CALPostHit *posthit);
  virtual ~S4CALPostHit();

  static void   SetSortBy (Char_t *opt="");
         Int_t  Compare   (const TObject*) const;
         Bool_t IsSortable()               const;

  inline Int_t         GetPreHitID      ()  const { return fPreHitID;        }
  inline Int_t         GetPostHitID     ()  const { return GetHitID();       }
  inline Double_t      GetEnergy        ()  const { return GetTotalEnergy(); }
  inline TVector3      GetInjectionPoint()  const { return GetPosition();    }
  inline S4CALPreHit*  GetCALPreHitPtr()    const { return fPreHitPtr;       }
  inline void          SetCALPreHitPtr(S4CALPreHit *p){fPreHitPtr = p; }

  void Print(Option_t *opt="") const;

 private:
  TString       fParticleName; //  Particle Name
  Int_t         fPreHitID;     //  pre hit id
  S4CALPreHit  *fPreHitPtr;    //! pre hit ptr

  ClassDef(S4CALPostHit, 2) // S4CALPreHit class
};

#endif
