////////////////////////////////////////////////////////////////////////
//
//  CAL PreHit class in ROOT
//
//  2004.10.16   Allister Sanchez  Original version.
//
// $Id: S4CALPreHit.cxx,v 1.9 2006/07/03 07:14:16 miyamoto Exp $
////////////////////////////////////////////////////////////////////////

#include <iostream>
#ifndef __HasJSFFromLCIO__
#include "J4CALPreHit.hh"
#include "J4CLXPreHit.hh"
#include "G4ParticleDefinition.hh"
#endif
#include "S4CALPreHit.h"
using namespace std;

ClassImp(S4CALPreHit);

//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit()
            : S4VExactHit(0,                 // hit ID
                          0,                 // track ID
                          0,                 // mother track ID
                          0,                 // PDG code
                          0,                 // charge
                          TVector3(0.,0.,0.),// position
                          0,                 // TOF
                          TVector3(0.,0.,0.),// momentum
                          0,                 // energy
                          0,                 // energy deposit
                          0),                // address ptr 
              fParticleName(""),             // particle name
              fPostHitID(0)
{
}


//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit(Int_t     postHitID,     // posthit ID
			 Int_t     preHitID,      // prehit ID
			 Int_t     trackID,       // track ID 
			 Int_t     motherTrackID, // mother track ID
			 Int_t     pcode,         // PDG code
			 Double_t  charge,        // charge
			 TVector3  pos,           // Injection position [cm]
			 Double_t  tof,           // TOF      [nsec]
			 TVector3  mom,           // momentum [GeV] 
			 Double_t  etot,          // energy
			 Double_t  edep,          // energy deposit [GeV]      
                         TVAddress *p)            // address ptr)
            : S4VExactHit(preHitID,              // hit ID
			  trackID,               // track ID 
			  motherTrackID,         // mother track ID
			  pcode,                 // PDG code
			  charge,                // charge
			  pos,                   // Injection position [cm]
			  tof,                   // TOF      [nsec]
			  mom,                   // momentum [GeV] 
			  etot,                  // energy
			  edep,                  // energy deposit [GeV] 
			  p),                    // address ptr)
              fParticleName(""),                 // particle name
              fPostHitID(postHitID)              // post hit ID
{
}

#ifndef __HasJSFFromLCIO__
//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit(J4CALPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetInjectionPoint().x()/cm, // position
                                   p->GetInjectionPoint().y()/cm,
                                   p->GetInjectionPoint().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetPostHitID())                       // post hit ID
{
}

//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit(J4CLXPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetInjectionPoint().x()/cm, // position
                                   p->GetInjectionPoint().y()/cm,
                                   p->GetInjectionPoint().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetPostHitID())                       // post hit ID
{
}
#endif

//--------------------------------------------------------------------
S4CALPreHit::~S4CALPreHit()
{
}

//--------------------------------------------------------------------
void S4CALPreHit::Print(Option_t *) const
{
  std::cout << "S4CALExactHit ================" << std::endl;
  std::cout << " PostHitID=" << GetPostHitID()
            << " PreHitID=" << GetPreHitID()
	    << " TrkID=" << GetTrackID()
	    << " TOF(ns)=" << GetTOF()
	    << " E(GeV)=" << GetTotalEnergy()
	    << " Name=" << fParticleName
	    << " PDGcode=" << GetPDGCode()
	    << " Charge=" << GetCharge() << std::endl;
  std::cout << " Momentum(GeV)=(" << GetMomentum().x() << ", "
	    << GetMomentum().y() << ", "
	    << GetMomentum().z() << ") " << std::endl;
  std::cout << " InjectionPoint(cm)=(" << GetInjectionPoint().x() << ", "
	    << GetInjectionPoint().y() << ", "
	    << GetInjectionPoint().z() << ") " << std::endl;
}

//---------------------------------------------------------------
Int_t S4CALPreHit::Compare(const TObject *hit) const
{
  return 0;
}

//---------------------------------------------------------------
void S4CALPreHit::SetSortBy(Char_t *opt)
{
}

//---------------------------------------------------------------
Bool_t S4CALPreHit::IsSortable() const
{
  return kFALSE;
}
