#ifndef __S4CDCMCTRUTHBUF__
#define __S4CDCMCTRUTHBUF__

//*************************************************************************
//* ----------------
//* S4VMCTruthBuf Class
//* ----------------
//* (Description)
//*	This set of classes reads ascii output of JUPITER and
//*	put them into root objects.
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4VMCTruthBuf		: steering class
//* (Update Record)
//*	2001/08/21  K.Hoshina	Original version.
//*************************************************************************

#include "S4VMCTruthBuf.h"

#include "S4CDCTypeDef.h"
#include "S4CDCMCTruth.h"

#ifdef __KUBORIE__
#include "TObject.h"
#include "TH2.h"
#include "TH1.h"
#include "TCanvas.h"
#endif

// ---------------
// MonteCarlo Truth Buffer
// ---------------

class S4CDCMCTruthBuf : public S4VMCTruthBuf {

public:
  S4CDCMCTruthBuf(const char *name ="S4CDCMCTruthBuf",
		  const char *title ="CDC MCTruth Buf", 
		  S4CDCMCTruth *mod =0,
		  Int_t bufsize = 20000);
  S4CDCMCTruthBuf(S4CDCMCTruth *mod, const char *name="S4CDCMCTruthBuf",
		  const char *title="CDC MCTruth Buf", Int_t bufsize=20000);
  virtual ~S4CDCMCTruthBuf();

  Bool_t ReadFile();
  void   RemoveDeltaRayHits();
  void   JoinSeparatedHits();
  inline virtual void SetEnergyDepositCut(Double_t cut)
                                { fEnergyDepositCut = cut; }
  inline virtual void SetRangeCut(Double_t cut)
                                { fRangeCut = cut; }

  TObjArray * GetTmpExactHitsPtr()            { return &fTmpExactHits; }
  TObjArray * GetInputBufPtr()                { return fInputBufPtr;   }
  void        SetInputBufPtr(TObjArray *bufp) { fInputBufPtr = bufp;   }

private:
  void   AppendTrack(TObjArray* tracktmpp);
  void   AppendCell(TObjArray* celltmpp);
  void   JoinHits(TObjArray* wiretmpp);
#ifdef __KUBORIE__
  void   InitHists();
#endif


private:
  Double_t  fEnergyDepositCut;	// energy cut for delta-ray
  Double_t  fRangeCut;		// range cut for delta-ray

  TObjArray fTmpExactHits;      //! very temporary!!!
  TObjArray *fInputBufPtr;      //! 

#ifdef __KUBORIE__
  struct THdef1 {
	Char_t    title[256];
	Int_t     nxbin;
	Double_t  xmin;
	Double_t  xmax;
  };
  struct THdef2 {
	Char_t    title[256];
	Int_t     nxbin;
	Double_t  xmin;
	Double_t  xmax;
	Int_t     nybin;
	Double_t  ymin;
	Double_t  ymax;
  };
  TH1D        **hLength;	//!
  TH2D        **hNHitE;		//!
  TH2D        **hNHitK;		//!
  TH2D        **hXY;		//!
  THdef1       *hLdef;		//!
  THdef2       *hNEdef;		//!
  THdef2       *hNKdef;		//!
  THdef2       *hXYdef;		//!

  static const  Int_t kNh;	//!
#endif
  
  ClassDef(S4CDCMCTruthBuf, 1) // CDC raw data buffer
};

#endif



