//*************************************************************************
//* --------------------
//* S4ITExactHit
//* --------------------
//* (Description)
//*	IT Hit class.
//*	Mother class : S4VExactHit
//*     
//* (Update Record)
//*	2004/08/07  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#ifndef __HasJSFFromLCIO__
#include "J4ITLayerHit.hh"
#endif

#include "S4ITExactHit.h"
#include "JSFSteer.h"

ClassImp(S4ITExactHit)

Int_t S4ITExactHit::fITExactHitSortBy = 0; 

//=====================================================================
//* constructor -------------------------------------------------------

S4ITExactHit::S4ITExactHit()
            : fLayerID(0), fLayerR(0.), fPhi(0.), fSide(0),
              fPrePosition(0.,0.,0.), fPostPosition(0.,0.,0.)
{
}

#ifndef __HasJSFFromLCIO__
//=====================================================================
S4ITExactHit::S4ITExactHit(const J4ITLayerHit *hitp)
            : S4VExactHit(0,                              // hit ID
                          hitp->GetTrackID(),             // track ID
                          hitp->GetMotherTrackID(),       // mother track ID
                          hitp->GetPDGEncoding(),         // particle ID
                          (Int_t)hitp->GetCharge(),       // charge
                          (1. / cm) * TVector3(hitp->GetPrePosition().x(),
                                               hitp->GetPrePosition().y(),
                                               hitp->GetPrePosition().z()),
                          hitp->GetTof(),                 // TOF
                          (1. / GeV) * TVector3(hitp->GetMomentum().x(),
                                                hitp->GetMomentum().y(),
                                                hitp->GetMomentum().z()),
                          hitp->GetTotalEnergy()   / GeV, // total energy
                          hitp->GetEnergyDeposit() / GeV, // energy deposit
                          0,                              // address ptr
                          0),                             // event ID 
              fLayerID(hitp->GetComponent()->GetMyID()), // pad raw No.
              fPrePosition((1./cm)*TVector3(hitp->GetPrePosition().x(),
                                            hitp->GetPrePosition().y(),
                                            hitp->GetPrePosition().z())),
              fPostPosition((1./cm)*TVector3(hitp->GetPostPosition().x(),
                                             hitp->GetPostPosition().y(),
                                             hitp->GetPostPosition().z()))
{
   TVector3 hitpos  = GetPosition();
   fLayerR          = hitpos.Pt();
   fPhi             = hitpos.Phi();
   static Int_t nb  = gJSF->Env()->GetValue("J4IT.NBarrel", 4);
   static Int_t ne  = gJSF->Env()->GetValue("J4iT.NEndcap", 7);
   fSide            = fLayerID < nb ? 0 : (fLayerID < ne ? 1 : -1);
}
#endif

S4ITExactHit::S4ITExactHit(Int_t     evtID,
                           Int_t     layerID, 
                           Int_t     trackID,
                           Int_t     mothertrackID,
                           Int_t     hitID, 
                           Int_t     pcode, 
                           Int_t     charge,
                           const TVector3 &pre, 
                           const TVector3 &post, 
                           const TVector3 &momentum,
                           Double_t  totE, 
                           Double_t  edep, 
                           Double_t  tof)
            : S4VExactHit(hitID, trackID,  mothertrackID, 
                          pcode, charge,   pre,
                          tof,   momentum, totE, 
                          edep,  0,        evtID),   
              fLayerID(layerID),
              fPrePosition(pre),
              fPostPosition(post)
{
   fLayerR = pre.Pt();
   fPhi    = pre.Phi();
   static Int_t nb  = gJSF->Env()->GetValue("J4IT.NBarrel", 4);
   static Int_t ne  = gJSF->Env()->GetValue("J4iT.NEndcap", 7);
   fSide            = fLayerID < nb ? 0 : (fLayerID < ne ? 1 : -1);
}


//=====================================================================
//* destructor --------------------------------------------------------

S4ITExactHit::~S4ITExactHit()
{
}

//=====================================================================
//* DebugPrint -----------------------------------------------------------

void S4ITExactHit::DebugPrint(const Char_t *) 
{
   using namespace std;

   std::cerr << "------<S4ITExactHit::DebugPrint>---------"  << std::endl
        << " hit ID          = "  << GetHitID()         << std::endl
        << " track ID        = "  << GetTrackID()       << std::endl
        << " mother track ID = "  << GetMotherTrackID() << std::endl
        << " pdg code        = "  << GetPDGCode()       << std::endl
        << " charge          = "  << GetCharge()        << std::endl
        << " position        = (" << GetPosition().X()  << ", "
                                  << GetPosition().Y()  << ", "
                                  << GetPosition().Z()  << ")" 
                                                        << std::endl
        << " momentum        = (" << GetMomentum().X()  << ", " 
                                  << GetMomentum().Y()  << ", " 
                                  << GetMomentum().Z()  << ")" 
                                                        << std::endl
        << " energy          = "  << GetTotalEnergy()   << std::endl
        << " tof             = "  << GetTOF()           << std::endl; 
}

