#ifndef __S4VMCTRUTH__
#define __S4VMCTRUTH__
//*****************************************************************************
//* ----------------
//* S4VMCTruth Class
//* ----------------
//*
//* (Description)
//*	This set of classes reads ascii output of JUPITER and
//*	put them into root objects.
//*
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*
//* (Provides)
//*	S4VMCTruthBuf		: raw data buffer
//*	S4VMCTruth		: steering class
//*
//* (Update Record)
//*	2001/08/21  K.Hoshina	Original version.
//*	2004/08/10  K.Fujii	Now handles input streams with static methods
//*                             so as to allow data with multiple detector
//*                             components.
//*
//*****************************************************************************

#include <string>

#include "JSFModule.h"

class S4VMCTruthBuf;

//-----------------------------------------------------------------------------
// ----------------
// Class S4VMCTruth
// ----------------
//
class S4VMCTruth : public JSFModule {

  friend class S4VMCTruthBuf;

  //---------------------------------------------------------------------------
  // =======
  // Methods
  // =======
  //
  // --------------
  // Public Methods
  // --------------
  //
  public:
     //------------------------------------------------------------------------
     // Ctors and dtor
     //---------------
     S4VMCTruth(const Char_t* name  = "MyMCTruth",
                const Char_t* title = "My MC Truth Module");

     virtual ~S4VMCTruth();

     //------------------------------------------------------------------------
     // Parent's methods to override
     //-----------------------------
     virtual  Bool_t BeginRun(Int_t nrun);
     virtual  Bool_t Process (Int_t ev);

     //------------------------------------------------------------------------
     // Utility functions
     //------------------
     virtual  void   DebugPrint(const Char_t *opt = "");

     static std::ifstream * GetInputStreamPtr();
     static void            SetInputFileName(const Char_t *name);

     static void            OpenInputStream();
     static void            CloseInputStream();

  //---------------------------------------------------------------------------
  // ============
  // Data Members
  // ============
  //
  private:
     static std::string     fgInputFileName;         //! input ascii file name 
     static std::ifstream  *fgInputStreamPtr;        //! input file stream
     static Int_t           fgEventToProcess;        //! event to process


     ClassDef(S4VMCTruth, 1) // Read ASCII data from Jupiter
};

#endif
