//*************************************************************************
//* ---------------------
//* S4BabyTrackHit Class
//* ---------------------
//* (Description)
//*	Class for a single Baby hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4BabyTrackHit		: Array containing "U4BabyHit"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*************************************************************************

#include <iostream>
#include <iomanip>
#include <math.h>
#include "S4BabyTrackHit.h"
#include "S4BabyExactHit.h"
#include "S4BabyHitWire.h"

ClassImp(S4BabyTrackHit)

//=====================================================================
//* constructor -------------------------------------------------------

S4BabyTrackHit::S4BabyTrackHit(S4BabyExactHit* exacthitp) 
	       : U4CDCTrackHit((S4BabyHit *)exacthitp->GetSmearedHitPtr()), 
                 fKilled(kFALSE),
                 fExactHitPtr(exacthitp),
                 fVectorHitPtr(0)
{
	if ( exacthitp->GetDriftLen() < 0 ) {
		SetLeftRight(-1);
	} else {
		SetLeftRight(+1);
	}
}

//=====================================================================
//* destructor --------------------------------------------------------

S4BabyTrackHit::~S4BabyTrackHit()
{
	
}

//=====================================================================
//* Compare -----------------------------------------------------------

Int_t S4BabyTrackHit::Compare(const TObject *obj) const
{
	Int_t wireno  = GetWireNumber();
	Int_t wirenoo = ((S4BabyTrackHit *)obj)->GetWireNumber();
   	if (wireno < wirenoo) return -1;
   	else if (wireno > wirenoo) return 1;
   	else return 0;
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4BabyTrackHit::DebugPrint(const Char_t *opt)
{
	std::cerr.setf(std::ios::right);
	std::cerr.setf(std::ios::fixed);
	std::cerr	<< "wire = (" << std::setw(1) << GetTrackID()
		<< "," << std::setw(1) << GetCellNumber()
		<< "," << std::setw(1) << GetWireNumber() << ")";
		
	GetHitPtr()->DebugPrint(opt);
	std::cerr.unsetf(std::ios::right);
	std::cerr.unsetf(std::ios::fixed);
}




