#ifndef __S4CALEMCLUSTERMAKERUTILS__
#define __S4CALEMCLUSTERMAKERUTILS__

//*************************************************************************   
//* ---------------------                      
//* S4CALEMClusterMakerUtils Class
//* ---------------------   
//* (Description) 
//*     represent calorimeter cluster
//* (Requires)
//*     S4CALHit
//* (Provides)  
//*     S4CALRealCluster            : made object class
//* (Update Record)
//*     2004/09/03  S.Yamamoto  Original version.
//*     2004/11/15  S.Yamamoto  Add IsOK
//*************************************************************************

#include "TVector3.h"
#include "S4CALSmallCluster.h"
#include "S4CALRealCluster.h"
#include "TObject.h"

#include <utility>
#include <map>
#include <vector>

class S4CALEMClusterMakerUtils {

 public:
  S4CALEMClusterMakerUtils();
  virtual ~S4CALEMClusterMakerUtils();

 public:

  //-----------------------------------------------------------------
  // Fit longitudinal profile of the shower. Fit function is
  // gamma distribution function.
  //-----------------------------------------------------------------
  std::vector<Int_t> EMFitter(std::map<Int_t,S4CALSmallCluster*> &sclmap,
			      S4CALSmallCluster* scls,
			      Double_t tubeR  = 5.,   // cm
			      Double_t tubeH  = 15.,  // cm
			      Double_t E0     = 5.,   // GeV
			      Int_t    maxitr = 100); // maximum iterations

  //-----------------------------------------------------------------
  // Calculate longitudinal energy weighted shower position and RMS
  // within ECAL region.
  //-----------------------------------------------------------------
  std::vector<Int_t> EMLongitudinalShape(std::map<Int_t,S4CALSmallCluster*> &sclmap,
					 S4CALSmallCluster* scls,
					 Double_t tubeR = 5.);   // cm

  //-----------------------------------------------------------------
  // Calculate cluster length
  //-----------------------------------------------------------------
  Double_t CalcClusterLength(S4CALRealCluster *cls, Double_t elimit = 0.8);
  
  //-----------------------------------------------------------------
  // Compare energy deposit between large and small tubes.
  // Tube radius is calculated based on Moliere radius.
  //-----------------------------------------------------------------
  std::vector<Int_t> EMLateralSpread(std::map<Int_t,S4CALSmallCluster*> &sclmap,
				     S4CALSmallCluster* scls,
				     Double_t multiplein  = 1.2,
				     Double_t multipleout = 2.4,
				     Double_t tubeH       = 20.);
  
  //-----------------------------------------------------------------
  // Compare energy deposit within ECAL region.
  //-----------------------------------------------------------------
  std::vector<Int_t> EECompare(std::map<Int_t,S4CALSmallCluster*> &sclmap,
			       S4CALSmallCluster* motherscl,
			       Double_t tubeR = 8.);
  
  //-----------------------------------------------------------------
  // Compare energy deposit between EM/HD region.
  //-----------------------------------------------------------------
  std::vector<Int_t> EHCompare(std::map<Int_t,S4CALSmallCluster*> &sclmap,
			       S4CALSmallCluster* motherscl,
			       Double_t tubeR = 15.,
			       Double_t tubeH = 60.);
  
  //=================================================================
  //* Getter & Setter
  //-----------------------------------------------------------------

 public:

  inline Int_t    GetMinNHits()         const { return fMinNHits;         }
  inline Int_t    GetNEMLayers()        const { return fNEMLayers;        }
  inline Double_t GetEMTotalRadLength() const { return fEMTotalRadLength; }

 public:

  inline void SetMinNHits(Int_t nhits)           { fMinNHits    = nhits;     }
  inline void SetNEMLayers(Int_t nlayers)        { fNEMLayers   = nlayers;   }
  inline void SetEMTotalRadLength(Double_t radl) { fEMTotalRadLength = radl; }

  //=================================================================
  //* Data member
  //-----------------------------------------------------------------

 public:

  static Int_t     fNEMLayers;         // number of layers in ECAL
  static Double_t  fEMTotalRadLength;  // total radiation length in ECAL
  static Int_t     fNHDLayers;         // number of layers in ECAL
  static Double_t  fHDTotalRadLength;  // total radiation length in ECAL
  Int_t     fMinNHits;


  ClassDef(S4CALEMClusterMakerUtils, 1)
};
    
#endif
