#ifndef __S4CALREALCLUSTER__
#define __S4CALREALCLUSTER__

//*************************************************************************
//* ---------------------
//* S4CALRealCluster Class
//* ---------------------
//* (Description)
//*	represent calorimeter cluster 
//* (Requires)
//*	S4CALHit
//* (Provides)
//*	S4CALRealCluster                   : made object class
//* (Update Record)
//*	2004/09/03  S.Yamamoto	Original version.
//*	2004/11/04  K.Fujii	Added drawable attribute.
//*************************************************************************

#include "S4VCluster.h"
#include "S4CALSmallCluster.h"


class S4CALRealCluster : public S4VCluster {

friend class S4CALRealClusterMakerBuf;

 public:
  S4CALRealCluster(S4HYBTrack      *trackp       = 0, 
		   EClusterType_t   clusterype   = kgCheated_cl, 
		   EParticleType_t  particletype = kgNotDecided_p);
  ~S4CALRealCluster();
  
  void 	DebugPrint(const Char_t *opt = "");
  
  // Data Getters
  Int_t      GetCALType(){ if (!fDone) Update(); return fCALType; }
  Double_t   GetR()        { if (!fDone) Update(); return fClusterR;}
  Double_t   GetTheta()    { if (!fDone) Update(); return fClusterTheta;}
  Double_t   GetPhi()      { if (!fDone) Update(); return fClusterPhi;}
  Double_t   GetTime()     { if (!fDone) Update(); return fTOF;}
  TVector3   GetPosition() { if (!fDone) Update(); return fPosition;}
  Double_t   GetEnergy()   { if (!fIsCalibrated) Calibrate(); return fEnergy;}
  Double_t   GetEnergyDeposit(){ if (!fDone) Update(); return fEnergyDeposit;}
  TVector3   GetMomentum() { if (!fIsCalibrated) Calibrate(); return fMomentum;}
  TObjArray &GetTracks()   { if (!fDone) Update(); return fTracks; }
  Double_t   GetFracEM()   { if (!fDone) Update(); return fFracEM; }

  EClusterType_t  GetClusterType()  { return fClusterType; }
  EParticleType_t GetParticleType() { return fParticleType; }   

  TVector3 GetThrustVector()   { if (!fDone) Update(); return fThrustVector;}
  Double_t GetThrust()         { if (!fDone) Update(); return fThrust;}
  Double_t GetBroadening()     { if (!fDone) Update(); return fBroadening;}
  Int_t    GetNHits()          { if (!fDone) Update(); return fNHits;}
  Double_t GetMeanLayerID(){ if(!fDone) Update(); return fMeanLayerID; }
  Int_t    GetMinLayerID() { if(!fDone) Update(); return fMinLayerID;  }
  Int_t    GetMaxLayerID() { if(!fDone) Update(); return fMaxLayerID;  }
  Int_t    GetMinELayerID(){ if(!fDone) Update(); return fMinELayerID; }
  Int_t    GetMaxELayerID(){ if(!fDone) Update(); return fMaxELayerID; }

  std::vector<S4CALHit*> GetCALHitsVector(){if(!fDone) Update(); return fCALHitsVector;}

  //===============================
  //* Service method
  //-------------------------------
  void        Add(const S4CALHit* hit);     // Add hit
  void        Add(S4CALSmallCluster* scl);  // Add scls containing hits
  void        AddMIPSmallCluster(S4CALSmallCluster* scl);
  void        AddTrackPtr(S4HYBTrack* trk){ if(trk) fTracks.Add(trk);}
  S4HYBTrack *GetTrackPtr(Int_t i=0);       // Get i-th track pointer

  void        Update();       // Update Real cluster information
  void        Calibrate();    // energy calibration

  //===================================
  //* Getter & Setter for Attached ID
  //-----------------------------------

  TVector3 GetILPosition()         { return fILPosition; }
  Double_t GetMIPEnergy()          { return fMIPEnergy;  }

  void  SetILPosition(const TVector3 pos)     { fILPosition  = pos; }
  void  SetMIPEnergy(const Double_t mip)      { fMIPEnergy   = mip; }
  void  SetClusterType(EClusterType_t type)   { fClusterType = type; }
  void  SetParticleType(EParticleType_t type);

  Bool_t IsPhoton()                { return fIsPhoton;    }
  Bool_t IsElectron()              { return fIsElectron;  }
  Bool_t IsCHD()                   { return fIsCHD;       }
  Bool_t IsNHD()                   { return fIsNHD;       }
  Bool_t IsMuon()                  { return fIsMuon;      }
  Bool_t IsNotDecided()            { return fIsNotDecided; }

  //=================================================================
  //* Data member
  //-----------------------------------------------------------------

 private:
  Double_t        fClusterR;         // Cluster Position(cm)
  Double_t        fClusterTheta;     // Cluster Position(radian)
  Double_t        fClusterPhi;       // Cluster Position(radian)
  TObjArray       fTracks;           // Cluster maching track array  
  Double_t        fFracEM;           // Energy fraction in EM

  EClusterType_t  fClusterType;      // Cluster Type
  EParticleType_t fParticleType;     // Particle Type
  Bool_t          fIsCalibrated;     // calibrated or not
  Bool_t          fIsPhoton;         // photon cluster
  Bool_t          fIsElectron;       // electron cluster
  Bool_t          fIsCHD;            // charged hadron cluster
  Bool_t          fIsNHD;            // neutral hadron cluster
  Bool_t          fIsMuon;           // muon cluster
  Bool_t          fIsNotDecided;     // particle type is not decided

  Int_t           fMinELayerID;      // minimam E-dep layer ID
  Int_t           fMaxELayerID;      // maximum E-dep layer ID

  TVector3    fILPosition;   // interaction layer position
  Double_t    fMIPEnergy;    // MIP part energy of this cluster

  static Int_t fRealClusterIDAll;
  static Int_t fNEvt;               // event number

  ClassDef(S4CALRealCluster, 1) // Cluster data information
};
    
#endif
