//*************************************************************************
//* --------------------
//* S4CDCHitMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad hits.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2001/08/22  K.Hoshina	Original version.
//*************************************************************************

#include <iostream>
#include "S4CDCHitMakerBuf.h"
#include "S4CDCExactHit.h"
#include "S4CDCHit.h"

#define __TMP__
//#define __DEBUG__

ClassImp(S4CDCHitMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4CDCHitMakerBuf::S4CDCHitMakerBuf(U4CDCTimeToXRelation *calib, const char *name,
	                           const char *title, S4CDCHitMaker *module,
	                           Int_t bufsize)
                : U4CDCHitMakerBuf(calib, name, title, module, bufsize),
                  fInputBufPtr(0)
{
}	       
	       
S4CDCHitMakerBuf::S4CDCHitMakerBuf(S4CDCHitMaker *module,
			           U4CDCTimeToXRelation *calib, 
			           const char *name,
	                           const char *title, 
	                           Int_t bufsize)
                : U4CDCHitMakerBuf(module, calib, name, title, bufsize),
                  fInputBufPtr(0)
{
}	       

//=====================================================================
//* destructor --------------------------------------------------------

S4CDCHitMakerBuf::~S4CDCHitMakerBuf()
{
}


//=====================================================================
//* MakeHits ----------------------------------------------------------

void S4CDCHitMakerBuf::MakeHits()
{
	if (!IsInitialized() && !fInputBufPtr) {
		std::cerr << "ERROR  S4CDCHitMakerBuf::MakeHits" << std::endl
		     << "...... cluster data not yet initialized." << std::endl;
		return;
	} else if (fDone) {
		return;
	}
	
	fDone = kTRUE;
		
	S4CDCExactHit::SetSortBy("CellID");
	fInputBufPtr->UnSort();
	fInputBufPtr->Sort();
	
	TObjArray layertmp(2000);
		
	Int_t layerlast = -9999;
		
	TIter next(fInputBufPtr);
	S4CDCExactHit *exacthitp;
	while ((exacthitp = (S4CDCExactHit *)next())) {
		Int_t layercur = exacthitp->GetLayerNumber();
		if ( layercur != layerlast ) {			
#ifdef __DEBUG__
			std::cerr << " S4CDCHitMakerBuf::MakeHits: LayerNumber = "
			     << layercur << std::endl;
#endif
			if ( layertmp.GetEntries() > 0 ) {
#ifdef __DEBUG__
			     std::cerr << " S4CDCHitMakerBuf::MakeHits: LayerNumber"
			          << layerlast << " hits end"   << std::endl;
#endif
				AppendLayer( &layertmp, &fOutputBuf );
				layertmp.Clear();
			}
			layerlast = layercur;
		} 
		layertmp.Add ( exacthitp );
	}
	
	if ( layertmp.GetEntries() > 0 ) {
		AppendLayer( &layertmp, &fOutputBuf);
		layertmp.Clear();
	}
	
}

//=====================================================================
//* AppendLayer -------------------------------------------------------

void S4CDCHitMakerBuf::AppendLayer(TObjArray* layertmpp, TObjArray* outputbufp)
{
	
	U4CDCHitLayer  *layerp = new U4CDCHitLayer();
	TObjArray       celltmp(1000);
	
//	S4CDCExactHit::SetSortBy("CellID");
//	layertmpp->Sort();
	
	Int_t celllast = -99999;
		
	TIter next(layertmpp);
	S4CDCExactHit *exacthitp;
	while ((exacthitp = (S4CDCExactHit *)next())) {
		Int_t cellcur = exacthitp->GetCellID();
		if ( cellcur != celllast ) {
#ifdef __DEBUG__
			std::cerr << " S4CDCHitMakerBuf::AppendLayer: CellID = "
			     << cellcur << std::endl;
#endif
			if ( celltmp.GetEntries() > 0 ) {
#ifdef __DEBUG__
			        std::cerr << " S4CDCHitMakerBuf::AppendLayer: CellID"
			             << celllast << " hits end"<< std::endl;
#endif
				AppendCell( &celltmp, layerp );
				celltmp.Clear();
			}
			celllast = cellcur;
		} 
		celltmp.Add ( exacthitp );
	}
	
	if ( celltmp.GetEntries() > 0 ) {
		AppendCell( &celltmp, layerp );
		celltmp.Clear();
	}
	
	if ( layerp->GetEntries() > 0 ) {
		outputbufp->Add(layerp);
	} else {
		delete layerp;
	}	
	
}

//=====================================================================
//* AppendCell --------------------------------------------------------

void S4CDCHitMakerBuf::AppendCell(TObjArray* celltmpp, U4CDCHitLayer* layercurp)
{

	U4CDCHitCell  *cellp  = 0;
	S4CDCHitWire  *wirep  = 0;
	
	S4CDCExactHit::SetSortBy("WireNumber");
	celltmpp->Sort();
	
	TIter next(celltmpp);
	S4CDCExactHit *exacthitp;
	while ((exacthitp = (S4CDCExactHit *)next())) {
		if (!cellp) {
			cellp  = new U4CDCHitCell();
#if 0
			wirep  = new S4CDCHitWire(fCalibPtr);
#else
			wirep  = new S4CDCHitWire();
#endif
		}
		S4CDCHit *ht = new S4CDCHit(exacthitp);
		Int_t r1 = wirep->Append(ht);
		switch (r1) {
			case -1: {		// 1st hit
				cellp->Append(wirep);
				layercurp->Append(cellp);
				break;
			}
			case  0: break; 	// appended
			case  3: {		// new layer
				std::cerr << " S4CDCHitMakerBuf::AppendCell:LogicError!!"
				     << " new layer not expected." << std::endl;
				break;
			}
			case  2: {		// new cell
				std::cerr << " S4CDCHitMakerBuf::AppendCell:LogicError!!"
				     << " new cell not expected." << std::endl;
				break;
			}
			case  1: {		// new wire
#ifdef __DEBUG__
			        std::cerr << " S4CDCHitMakerBuf::AppendCell: WireNumber = "
			             << wirep->GetWireNumber() << " "
			             << wirep->GetEntries() << " hits" << std::endl;	 
#endif       
			        wirep->KillHits();
#if 0
				wirep = new S4CDCHitWire(fCalibPtr);
#else
				wirep = new S4CDCHitWire();
#endif
				wirep->Append(ht);
				cellp->Append(wirep);
				break;
			}
		}
	}
	if ( wirep && !wirep->IsKilled() ) wirep->KillHits();
#ifdef __DEBUG__
        std::cerr << " S4CDCHitMakerBuf::AppendCell: WireNumber = "
	     << wirep->GetWireNumber() << " "
             << wirep->GetEntries() << " hits" << std::endl;	        
#endif       
}
