//*************************************************************************
//* --------------------
//* S4PFOMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes Particle Flow Objects.
//* (Requires)
//*     JSFEventBuf
//* (Provides)
//*     S4PFOMakerBuf           : buffering class
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include <iostream>
#include "JSFSteer.h"
#include "S4CALCluster.h"
#include "S4PFObject.h"
#include "S4PFOFilter.h"
#include "S4KinkFinder.h"
#include "S4V0Finder.h"
#include "S4PFOMakerBuf.h"
#include "S4CALClusterMakerBuf.h"
#include "S4HYBTrackMakerBuf.h"

#if 1
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4CALTypeDef.h"
#include "S4TPCMCTruthBuf.h"
#include "S4GENMCTruthBuf.h"
#include "S4PrimaryParticle.h"
#include "S4PrimaryVertex.h"
#endif


//#define __DEBUG__
//#define __ACCEPTANCE_HOLE__
//

Bool_t S4PFOMakerBuf::fgNoKinkFinder = kFALSE;
Int_t  S4PFOMakerBuf::fgKinkScheme   = 0;

ClassImp(S4PFOMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------

S4PFOMakerBuf::S4PFOMakerBuf(S4PFOMaker   *module,
                             const Char_t *name, 
			     const Char_t *title, 
                             Int_t         bufsize)
		    : JSFEventBuf(name, title, (JSFModule*)module),
		      fDone(kFALSE),
                      fInputDataBufPtr(0),
                      fOutputBufPtr(0),
                      fBufSize(bufsize),
                      fFilterPtr(0),
                      fKinkFinderPtr(0),
                      fV0FinderPtr(0)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4PFOMakerBuf::~S4PFOMakerBuf()
{
   if (fOutputBufPtr) fOutputBufPtr->Delete();
   delete fOutputBufPtr;
}

//===================================================================== 
//* GetInputBufPtr ----------------------------------------------------       

JSFEventBuf* S4PFOMakerBuf::GetInputDataBufPtr()
{
  return fInputDataBufPtr;
}

//=====================================================================
//* GetOutputBufPtr ---------------------------------------------------

TObjArray* S4PFOMakerBuf::GetOutputBufPtr()
{
   return fOutputBufPtr;
}

//=====================================================================
//* GetEntries --------------------------------------------------------

Int_t S4PFOMakerBuf::GetEntries()
{
   if (!fDone) MakePFOs(); 
   return fOutputBufPtr->GetEntries();
}

//=====================================================================
//* SetInputDataBuf ---------------------------------------------------

void S4PFOMakerBuf::SetInputDataBufPtr(JSFEventBuf *buf)
{
   fDone = kFALSE;
   fInputDataBufPtr = buf;
   if (!fOutputBufPtr) fOutputBufPtr = new TObjArray(fBufSize);
   if (fOutputBufPtr->GetEntries() > 0) fOutputBufPtr->Delete();
}

//=====================================================================
//* SetPFOFilterPtr ---------------------------------------------------

void S4PFOMakerBuf::SetPFOFilterPtr(S4PFOFilter *pfofp)
{
  if (pfofp && pfofp == fFilterPtr) return;
  delete fFilterPtr;
  if (!pfofp) {
     fFilterPtr = new S4PFOFilter();
  } else {
     fFilterPtr = pfofp;
  }
}

//=====================================================================
//* SetKinkFinderPtr --------------------------------------------------

void S4PFOMakerBuf::SetKinkFinderPtr(S4KinkFinder *kinkfp)
{
  if (kinkfp && kinkfp == fKinkFinderPtr) return;
  delete fKinkFinderPtr;
  if (!kinkfp) {
     fKinkFinderPtr = new S4KinkFinder();
  } else {
     fKinkFinderPtr = kinkfp;
  }
  cerr << ">>> S4PFOMakerBuf::SetKinkFinder to " << fKinkFinderPtr << endl;
}


//=====================================================================
//* SetV0FinderPtr ----------------------------------------------------

void S4PFOMakerBuf::SetV0FinderPtr(S4V0Finder *v0fp)
{
  if (v0fp && v0fp == fV0FinderPtr) return;
  delete fV0FinderPtr;
  if (!v0fp) {
     fV0FinderPtr = new S4V0Finder();
  } else {
     fV0FinderPtr = v0fp;
  }
  cerr << ">>> S4PFOMakerBuf::SetV0Finder to " << fV0FinderPtr << endl;
}

//=====================================================================
//* SetNoKinkFinder ---------------------------------------------------

void S4PFOMakerBuf::SetNoKinkFinder(Bool_t set)
{
  fgNoKinkFinder = set;
  cerr << "S4PFOMakerBuf::SetNoKinkFinder() :::::::::::::::" << endl
       << "   Kink finder has been switched off! "           << endl;
}

//=====================================================================
//* SetKinkScheme -----------------------------------------------------

void S4PFOMakerBuf::SetKinkScheme(Int_t set)
{
  fgKinkScheme = set;
  cerr << "S4PFOMakerBuf::SetKinkScheme() :::::::::::::::::" << endl
       << "  Use " << (set ? "kink daughters":"kink mother") << endl;
}

//=====================================================================
//* UpdatePFOs --------------------------------------------------------
 
void S4PFOMakerBuf::UpdatePFOs()
{
   if (!fFilterPtr) SetPFOFilterPtr();
   if (!fDone) MakePFOs();
   if (!fOutputBufPtr->GetEntries()) return;
   TIter next(fOutputBufPtr);
   S4PFObject *pfp = 0;
   while ((pfp = static_cast<S4PFObject *>(next()))) {
      pfp->Update();
      if (!fFilterPtr->IsOK(*pfp)) pfp->Lock();
   }
}

//=====================================================================
//* DebugPrint --------------------------------------------------------
 
void S4PFOMakerBuf::DebugPrint(const Char_t *)
{
   using namespace std;
   using namespace PID;

   if (fOutputBufPtr && fOutputBufPtr->GetEntries()) {
      TIter next(fOutputBufPtr);
      S4PFObject *pfp = 0;
      while ((pfp = static_cast<S4PFObject *>(next()))) {
         S4CALCluster *cp  = pfp->GetClusterPtr();
         if (cp) {
            S4CALPreHit  *prep = static_cast<S4CALExactHit *>(
                                    static_cast<S4CALHit *>(
                                       cp->At(0)
                                    )->GetExactHitPtr()
                                 )->GetPreHitPtr();
            S4TPCPostHit *posp = prep->GetPostHitPtr();
            S4VExactHit  *php  = static_cast<S4VExactHit *>(posp)
                               ? static_cast<S4VExactHit *>(posp)
                               : static_cast<S4VExactHit *>(prep);

            Double_t Epr  = php->GetTotalEnergy();
            Double_t Ecl  = cp->GetEnergy();
            Bool_t   isEM = pfp->GetPID() == kgElectron_id 
                            || pfp->GetPID() == kgPhoton_id;
            // >>>>>>>>>>>>>>>>> TEMPORARY >>>>>>>>>>>>>>>>>>>>>
            Double_t sig  = (isEM ? 0.2 : 0.5)/TMath::Sqrt(Epr);
            static Double_t nsig = 4.;
            // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

            if (TMath::Abs(Ecl - Epr) > nsig*sig*Epr
                && pfp->GetPID() != kgMuon_id) {
               cerr << " >>>>>> S4PFOMakerBuf::DebugPrint >>>>>>>>" << endl
                    << "   Too big discrepancy "                    << endl
                    << "   Epre = " << Epr     << " [GeV]"          << endl
                    << "   Ecls = " << Ecl     << " [GeV]"          << endl
                    << " >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>" << endl;
               pfp->DebugPrint("Detailed");
            } else {
	      pfp->DebugPrint();
            }
         } else {
	   pfp->DebugPrint();
         }
      }
   }
}

//=====================================================================
//* MakePFOs ----------------------------------------------------------

void S4PFOMakerBuf::MakePFOs()
{
   if (fDone) return; // check if pfos already made 

   SetPrimaryBPs();

   fDone = kTRUE;
   fOutputBufPtr->SetOwner();        // buffer owns PFOs 
   // make cluster pfo
   TObjArray &calClusters = * static_cast<S4CALClusterMakerBuf *>(GetInputDataBufPtr())->GetOutputBufPtr();
   if (calClusters.GetEntries() > 0) {
      TIter next(&calClusters); // prepare cluster iterator
      S4CALCluster *cp = 0;
      while ((cp = static_cast<S4CALCluster *>(next()))) {
         fOutputBufPtr->Add(new S4PFObject(cp));
      } 
   } 
   // make track pfo
   S4HYBTrackMaker *htmp 
         = static_cast<S4HYBTrackMaker *>(gJSF->FindModule("S4HYBTrackMaker"));
   S4HYBTrackMakerBuf *htmbp = static_cast<S4HYBTrackMakerBuf *>(htmp->EventBuf());
   TObjArray &hybTracks = *htmbp->GetOutputBufPtr();
   if (hybTracks.GetEntries() > 0) {
      TIter next(&hybTracks); // prepare track iterator
      S4HYBTrack *tp = 0;
      while ((tp = dynamic_cast<S4HYBTrack *>(next()))) {
        if (!tp->IsLocked()) fOutputBufPtr->Add(new S4PFObject(tp));
      }
   }

   // Find kinks
   if (!fgNoKinkFinder) {
     if (!fKinkFinderPtr) SetKinkFinderPtr();
     fKinkFinderPtr->FindKinks(fgKinkScheme);
   }

   // Find V0s if any V0 finder there
   if (fV0FinderPtr) fV0FinderPtr->FindV0s();

#ifdef __ACCEPTANCE_HOLE__
   // make acceptance hole pfo
   TIter nextpbp(fPrimaryBPs);
   S4BreakPoint *pbpp;
   while ((pbpp = static_cast<S4BreakPoint *> (nextpbp()))) {
     if (!pbpp->IsLocked()) {
       fOutputBufPtr->Add(new S4PFObject(pbpp));
     }
   }
#endif
#ifdef __DEBUG__
   DebugPrint();
#endif
}

//=====================================================================       
//* SetPrimaryBPs -----------------------------------------------------        

void S4PFOMakerBuf::SetPrimaryBPs()
{
   fPrimaryBPs.Clear();
   S4PrimaryVertex *pvp = static_cast<S4PrimaryVertex *>(
                            static_cast<S4GENMCTruthBuf *>(
                              gJSF->FindModule("S4GENMCTruth")->EventBuf()
	                    )->GetOutputBufPtr()->At(0)
                          );
   TIter next(pvp->GetParticles());
   S4PrimaryParticle *ppp;
   while ((ppp = static_cast<S4PrimaryParticle *>(next()))) {
     if (ppp->GetTrackID() >= 0 && !ppp->GetDaughters()->GetEntries()) { // primary with no daughters
       Int_t j4tkid = ppp->GetTrackID();
       S4BreakPoint *bpp = S4BreakPoint::GetBreakPointPtr(j4tkid);
       if (bpp) {
         bpp->SetBit(BIT(20));		// flag this as a primary BP
         fPrimaryBPs.Add(bpp);		// store this in the primary BP TObjArray
       } else {
         cerr << ">>>>>> Error in S4PFOMakerBuf::GetPrimaryBPs() " << endl
              << " bpp = 0! unexpected! abort.                   " << endl;
         ppp->Print();
         abort();
       }
     }
   }
}
