#ifndef __S4PFOMAKERBUF__
#define __S4PFOMAKERBUF__

//*************************************************************************
//* ---------------------
//* S4PFOMakerBuf Class
//* ---------------------
//* (Description)
//*	This set of classes makes Particle Flow Objects. 
//* (Requires)
//*	JSFEventBuf
//* (Provides)
//*	S4PFOMakerBuf		: buffering class
//* (Update Record)
//*	2004/09/03  S.Yamamoto	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "S4PFOMaker.h"

class S4PFOFilter;
class S4KinkFinder;
class S4V0Finder;
 
class S4PFOMakerBuf : public JSFEventBuf {
friend class S4PFOMaker;

public:
  S4PFOMakerBuf(S4PFOMaker   *mod     = 0,
                const Char_t *name    = "S4PFOMakerBuf",
                const Char_t *title   = "Particle Flow Object Buffer", 
                      Int_t   bufsize = 50);
  virtual ~S4PFOMakerBuf();

  virtual JSFEventBuf  *GetInputDataBufPtr();  // returns ptr to input buf
  virtual TObjArray    *GetOutputBufPtr   ();  // returns pointer to pfo buffer 
  virtual Int_t	        GetEntries        ();  // returns number of pfos
  virtual void          SetInputDataBufPtr(JSFEventBuf  *buf);         // set ptr to input buf
  virtual void          SetPFOFilterPtr   (S4PFOFilter  *filter = 0);  // set ptr to pfo filter
  virtual void          SetKinkFinderPtr  (S4KinkFinder *finder = 0);  // set ptr to kink finder
  virtual void          SetV0FinderPtr    (S4V0Finder   *finder = 0);  // set ptr to v0 finder
  virtual S4KinkFinder *GetKinkFinderPtr  () { return fKinkFinderPtr; }// get ptr to kink finder
  virtual S4V0Finder   *GetV0FinderPtr    () { return fV0FinderPtr;   }// get ptr to v0 finder

  virtual void          UpdatePFOs        ();
  virtual void 	        DebugPrint        (const Char_t *opt = "");  
  virtual TObjArray    &GetPrimaryBPs     () { return fPrimaryBPs; }

  static  Bool_t        GetNoKinkFinder   () { return fgNoKinkFinder; }
  static  void          SetNoKinkFinder   (Bool_t set = kTRUE);
  static  Int_t         GetKinkScheme     () { return fgKinkScheme; }
  static  void          SetKinkScheme     (Int_t set = 1);

private:  
  virtual void	        MakePFOs(); 	     // Make PFOs
  virtual void          SetPrimaryBPs();     // Flag primary breakpoints
//--------------------------------------------------------------------------
private:
  Bool_t         fDone;                  //  (1,0) = (PFOMake done, not yet)
  JSFEventBuf   *fInputDataBufPtr;       //  Input data buffer 
  TObjArray     *fOutputBufPtr;          //  Output buffer ptr
  Int_t          fBufSize;               //  Output buffer size
  S4PFOFilter   *fFilterPtr;             //! PFO filter
  S4KinkFinder  *fKinkFinderPtr;         //! Kink finder
  S4V0Finder    *fV0FinderPtr;           //! V0 finder
  TObjArray      fPrimaryBPs;            //  primary breakpoint array

  static Bool_t fgNoKinkFinder;          //! switch off kink finder if true
  static Int_t  fgKinkScheme;            //! (0,1) = (mother, daughter)

  ClassDef(S4PFOMakerBuf, 1) // PFO data buffer
};

#endif
