//*************************************************************************
//* ---------------------
//* S4TPCTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4TPCTrackHit"s for a single track.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4TPCTrack		: Array containing "S4TPCTrackHit"s
//* (Update Record)
//*	2004/08/26  A.Yamaguchi    Original Ver.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "S4TPCTrack.h"
#include "S4TPCTrackHit.h"
#include "TPolyMarker3D.h"
#include "TVirtualPad.h"
//#define __DEBUG__

ClassImp(S4TPCTrack)

//=====================================================================
//* constructor -------------------------------------------------------

S4TPCTrack::S4TPCTrack()
{
#ifdef __DEBUG__
   std::cerr << "constructor for S4TPCTrack is called..." << std::endl;
#endif
   SetOwner();
}

S4TPCTrack::S4TPCTrack(S4TPCTrackMaker* trkm)
          : U4TPCTrack(trkm)
{
#ifdef __DEBUG__
   std::cerr << "constructor for S4TPCTrack is called..." << std::endl;
#endif
   SetOwner();
}

#if 0
S4TPCTrack::S4TPCTrack(const S4TPCTrack &tk)
{
   SetOwner();
}
#endif

//=====================================================================
//* destructor --------------------------------------------------------

S4TPCTrack::~S4TPCTrack()
{
}

//=====================================================================
//* GetPDGCode --------------------------------------------------------

Int_t S4TPCTrack::GetPDGCode()
{
   S4TPCTrackHit *thp = (S4TPCTrackHit *)((TObjArray *)this)->operator[](0);
   S4TPCHit   *php = (S4TPCHit *)thp->GetHitPtr();

   return php->GetExactHitPtr()->GetPDGCode();
}

//=====================================================================
//* GetTrackID --------------------------------------------------------

Int_t S4TPCTrack::GetTrackID()
{
   S4TPCTrackHit *thp = (S4TPCTrackHit *)((TObjArray *)this)->operator[](0);
   S4TPCHit   *php = (S4TPCHit *)thp->GetHitPtr();
#if 0
   std::cerr << "tptrk = " << (void *)this << " j4trkid = " << php->GetExactHitPtr()->GetTrackID() << std::endl;
#endif

   return php->GetExactHitPtr()->GetTrackID();
}

//=====================================================================
//* GetMotherTrackID --------------------------------------------------

Int_t S4TPCTrack::GetMotherTrackID()
{
   S4TPCTrackHit *thp = (S4TPCTrackHit *)((TObjArray *)this)->operator[](0);
   S4TPCHit   *php = (S4TPCHit *)thp->GetHitPtr();

   return php->GetExactHitPtr()->GetMotherTrackID();
}

//=====================================================================
//* Append ------------------------------------------------------------

Bool_t S4TPCTrack::Append(S4TPCHit *hitp)
{
#if 0
   if (hitp->IsLocked()) return kFALSE;
   Int_t trackid    = hitp->GetExactHitPtr()->GetTrackID();
   Int_t curtrackid = GetEntries()
            ? static_cast<S4TPCTrackHit *>(At(0))->GetHitPtr()
              ->GetExactHitPtr()->GetTrackID()
            : trackid;
   if (curtrackid == trackid) {
      Add(new S4TPCTrackHit(hitp));
      return kTRUE;
   } else {
      return kFALSE;
   }
#else
   Int_t trackid    = hitp->GetExactHitPtr()->GetTrackID();
   Int_t curtrackid = GetEntries()
            ? static_cast<S4TPCTrackHit *>(At(0))->GetHitPtr()
              ->GetExactHitPtr()->GetTrackID()
            : trackid;
   if (curtrackid == trackid) {
      S4TPCTrackHit *trackhitp = new S4TPCTrackHit(hitp);
      Add(trackhitp);
      if (hitp->IsLocked()) trackhitp->Lock();
      return kTRUE;
   } else {
      return kFALSE;
   }
#endif
} 

//=====================================================================
//* Verify ------------------------------------------------------------

Bool_t S4TPCTrack::Verify()
{
   if (GetEntries() < 4) return kFALSE;

   S4TPCTrackHit::SetSortBy("Time");
   UnSort();
   Sort();  
#if 1
   Int_t  lastlayer = -9999;
   Int_t firstlayer = static_cast<S4TPCTrackHit *>(At(0))
                      ->GetHitPtr()->GetExactHitPtr()->GetPadRowNumber();
   Int_t secondlayer = static_cast<S4TPCTrackHit *>(At(1))
                      ->GetHitPtr()->GetExactHitPtr()->GetPadRowNumber();
   if (secondlayer <= firstlayer) return kFALSE;
#if 1
   Int_t layermin =  99999;
   Int_t layermax = -99999;
#endif
   Int_t curlayer = -9999;
   Bool_t islooping = kFALSE; 
   static const Int_t maxlayer = GetTrackMakerPtr()->GetDetectorPtr()->GetNmeasPoints() - 1;
   TIter next(this);
   S4TPCTrackHit *trackhitp;
   while ((trackhitp = static_cast<S4TPCTrackHit *>(next()))) {
      //if (trackhitp->IsLocked()) continue;
      S4TPCExactHit *exacthitp = trackhitp->GetHitPtr()->GetExactHitPtr();

      lastlayer = curlayer;
      curlayer = exacthitp->GetPadRowNumber();
#if 1
      if (curlayer > layermax) layermax = curlayer;
      if (curlayer < layermin) layermin = curlayer;
#endif
      if (curlayer <= lastlayer) { 
         if (!curlayer || curlayer == maxlayer) {
            trackhitp->Lock();
            break;
         }
         if (!islooping) islooping = kTRUE;
      } else {
         if (islooping) {
            trackhitp->Lock();
            break;
         } 
      }
   }
#if 1
   // Reject a track having less than 10 layers
   if ((layermax-layermin) < 10) return kFALSE;
#endif
   
   while ((trackhitp = static_cast<S4TPCTrackHit *>(next()))) trackhitp->Lock(); 
#else
   static const Int_t nlayers = gJSF->Env()->GetValue("J4TPC.NLayers", 200);
   Bool_t isfwd = kTRUE;

   TIter next(this);
   S4TPCTrackHit *thitp = 0;
   while ((thitp = static_cast<S4TPCTrackHit *>(next()))) {
      S4TPCExactHit *ehitp = thitp->GetHitPtr()->GetExactHitPtr();
      if (ehitp->GetPadRowNumber() == nlayers - 1) break;

      TVector3 pre  = ehitp->GetPrePosition();
      TVector3 post = ehitp->GetPostPosition();
      if (isfwd) {
         if (pre.Perp2() > post.Perp2()) isfwd = kFALSE;
      } else {
         if (pre.Perp2() < post.Perp2()) {
            thitp->Lock();
            break;
         }
      }
   }
 
   while ((thitp = static_cast<S4TPCTrackHit *>(next()))) thitp->Lock();
#endif
   return !(GetNhits() < 4);
}

//=====================================================================
//* Draw --------------------------------------------------------------

void S4TPCTrack::Draw(Int_t color, const Char_t * /* opt */)
{
   if (!gPad) return;
   gPad->cd();

   TPolyMarker3D *pm3dp = new TPolyMarker3D(this->GetEntries());
   pm3dp->SetBit(kCanDelete);
   pm3dp->SetMarkerColor(color);

   Int_t nhits = 0;
   TIter next(this);
   S4TPCTrackHit *trkhitp = 0;
   while ((trkhitp = static_cast<S4TPCTrackHit *>(next()))) { 
      //if (trkhitp->IsLocked()) continue;
      TVector3 pos = trkhitp->GetHitPtr()->GetExactHitPtr()->GetPosition();
      pm3dp->SetPoint(nhits, pos.X(), pos.Y(), pos.Z());
      nhits++;
   }
   pm3dp->Draw();
   gPad->Update();
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void  S4TPCTrack::DebugPrint(const Char_t  *opt)
{
   U4TPCTrack::DebugPrint(opt);
}
