#ifndef __S4VTXHIT__
#define __S4VTXHIT__
//*************************************************************************
//* ---------------------
//* S4VTXHit Class
//* ---------------------
//* (Description)
//*	Class for a single VTX hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	S4VTXHit		: TObject containing single hit data
//* (Update Record)
//*	2004/12/15  T.Fujikawa	Original version.
//*************************************************************************

#include "S4VExactHit.h"
#include "S4VTXExactHit.h"
#include "U4VTXHit.h"

class S4VTXHit : public U4VTXHit {

public:
  S4VTXHit(S4VTXExactHit * exacthit);
  
  //==========================================================
  // --- ATTENTION ------
  // Both copy constructor and operator= don't set "this" pointer
  // to their ExactHit's Smeared Hit Pointer.
  //
     inline       S4VTXHit(const S4VTXHit & right);
     inline const S4VTXHit& operator=(const S4VTXHit& right);
  //==========================================================
  
     virtual ~S4VTXHit();

  // Getter --------------------------------------------------------------- 
  inline S4VTXExactHit *GetExactHitPtr() const { return fExactHitPtr;       }
  
  // Setter --------------------------------------------------------------- 
  static void  SetSortBy(Char_t *opt = "TrackID");
  static void  SetSigRPhi( Double_t sg=0.0 ) { fgSigDRPhi = sg; }
  static void  SetSigZ( Double_t sg=0.0 ) { fgSigDZ = sg ; }


  // other method --------------------------------------------------------- 
  inline Bool_t  IsSortable() const { return kTRUE;            }

  virtual Int_t  Compare(const TObject *obj) const;
  void           DebugPrint(const Char_t * opt = "Brief");
  void   Update();

  static void Print();

private:
  virtual void   CalculatePosition();

private:

  static Int_t     fgSortBy;           // sort flag
  static Double_t  fgSigDRPhi;        // 
  static Double_t  fgSigDZ;           // 
  S4VTXExactHit   *fExactHitPtr;               // Pointer to Exact Hit 

  ClassDef(S4VTXHit, 1)                        // VTX Anode Hit
  
};

//=====================================================================
//---------------------------------
// inline function for S4VTXHit
//---------------------------------

inline S4VTXHit::S4VTXHit(const S4VTXHit& right)
{
   fExactHitPtr        = right.fExactHitPtr;
   
   // !!! Don't set this to S4VTXExactHit::SmearedHitPointer !!! 
}

inline const S4VTXHit& S4VTXHit::operator= (const S4VTXHit& right)
{            
   fExactHitPtr        = right.fExactHitPtr;
   
   // !!! Don't set this to S4VTXExactHit::SmearedHitPointer !!! 
                  
     return *this;
}


#endif
