//*************************************************************************
//* -----------
//* U4BabyAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze CDC data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "U4BabyAnalysis.h"
#include <iostream>
const Int_t U4BabyAnalysis::kZoneX   = 4;
const Int_t U4BabyAnalysis::kZoneY   = 3;
const Int_t U4BabyAnalysis::kNh = kZoneX*kZoneY;

ClassImp(U4BabyAnalysis)


//___________________________________________________________________________
U4BabyAnalysis::U4BabyAnalysis(const Char_t *name, const Char_t *title)
	: U4VAnalysis(name, title), fMode(0), fNwires(5), 
	  xChi2(5.), xNsigmas(3.), xPh(9999999.), 
	  xSclHodSum(5000), xX7lo(-9999.), xX7hi(9999.),
	  fTk1st(0), fTklast(99999)
{
   hdefr = new THdef2[kNh];
   hdefs = new THdef2[kNh];
   hdefp = new THdef2[kNh];
   hdefz = new THdef2[kNh];
   hdefe = new THdef1[kNh];
   hdefw = new THdef2[kNh];
   hdefg = new THdef2[kNh];
   hdeft = new THdef1[kNh];
   hdefu = new THdef2[kNh];
   hdefa = new THdef2[kNh];
   hdefn = new THdef2[kNh];
   hdefh = new THdef2[kNh];
   hdef7 = new THdef2[kNh];
   
   Char_t st[256], std[256];
   Int_t i;
   for (i = 0; i < kNh; i++) {
      sprintf(st,"U4BabyAnalysis.hRes%2.2d",i);
      sprintf(std,"Res%2.2d 50,0.,5.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefr[i].title,
			&hdefr[i].nxbin, &hdefr[i].xmin, &hdefr[i].xmax,
			&hdefr[i].nybin, &hdefr[i].ymin, &hdefr[i].ymax);
      sprintf(st,"U4BabyAnalysis.hPh%2.2d",i);
      sprintf(std,"Ph%2.2d 50,0.,5.,200,0.,10000.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefs[i].title,
			&hdefs[i].nxbin, &hdefs[i].xmin, &hdefs[i].xmax,
			&hdefs[i].nybin, &hdefs[i].ymin, &hdefs[i].ymax);
      sprintf(st,"U4BabyAnalysis.hPhPk%2.2d",i);
      sprintf(std,"PhPk%2.2d 50,0.,5.,256,0.,256.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefp[i].title,
			&hdefp[i].nxbin, &hdefp[i].xmin, &hdefp[i].xmax,
			&hdefp[i].nybin, &hdefp[i].ymin, &hdefp[i].ymax);
      sprintf(st,"U4BabyAnalysis.hPed%2.2d",i);
      sprintf(std,"Ped%2.2d 50,0.,5.,256,0.,256.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefz[i].title,
			&hdefz[i].nxbin, &hdefz[i].xmin, &hdefz[i].xmax,
			&hdefz[i].nybin, &hdefz[i].ymin, &hdefz[i].ymax);
      sprintf(st,"U4BabyAnalysis.hEff%2.2d",i);
      sprintf(std,"Eff%2.2d 10,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",
      			 hdefe[i].title,
	                &hdefe[i].nxbin, &hdefe[i].xmin, &hdefe[i].xmax);
      sprintf(st,"U4BabyAnalysis.hTwk%2.2d",i);
      sprintf(std,"Twk%2.2d 256,0.,256.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefw[i].title,
			&hdefw[i].nxbin, &hdefw[i].xmin, &hdefw[i].xmax,
			&hdefw[i].nybin, &hdefw[i].ymin, &hdefw[i].ymax);
      sprintf(st,"U4BabyAnalysis.hSgt2%2.2d",i);
      sprintf(std,"Sgt2%2.2d 50,0.,5.,100,0.,0.0001",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefg[i].title,
			&hdefg[i].nxbin, &hdefg[i].xmin, &hdefg[i].xmax,
			&hdefg[i].nybin, &hdefg[i].ymin, &hdefg[i].ymax);
      sprintf(st,"U4BabyAnalysis.hEff2tk%2.2d",i);
      sprintf(std,"Eff2tk%2.2d 50,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",
      			 hdeft[i].title,
	                &hdeft[i].nxbin, &hdeft[i].xmin, &hdeft[i].xmax);
      sprintf(st,"U4BabyAnalysis.hSepRes%2.2d",i);
      sprintf(std,"SepRes%2.2d 50,0.,5.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefu[i].title,
			&hdefu[i].nxbin, &hdefu[i].xmin, &hdefu[i].xmax,
			&hdefu[i].nybin, &hdefu[i].ymin, &hdefu[i].ymax);
      sprintf(st,"U4BabyAnalysis.hTC2Sep%2.2d",i);
      sprintf(std,"TC2Sep%2.2d 128,0.,2048.,50,0.0,5.0",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefa[i].title,
			&hdefa[i].nxbin, &hdefa[i].xmin, &hdefa[i].xmax,
			&hdefa[i].nybin, &hdefa[i].ymin, &hdefa[i].ymax);
      sprintf(st,"U4BabyAnalysis.hSumw5%2.2d",i);
      sprintf(std,"Sumw5%2.2d 50,0.,5.,200,0.,10000.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefn[i].title,
			&hdefn[i].nxbin, &hdefn[i].xmin, &hdefn[i].xmax,
			&hdefn[i].nybin, &hdefn[i].ymin, &hdefn[i].ymax);
      sprintf(st,"U4BabyAnalysis.hHodX%2.2d",i);
      sprintf(std,"HodX%2.2d 100,0.,100000.,1000,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefh[i].title,
			&hdefh[i].nxbin, &hdefh[i].xmin, &hdefh[i].xmax,
			&hdefh[i].nybin, &hdefh[i].ymin, &hdefh[i].ymax);
      sprintf(st,"U4BabyAnalysis.hEvtX%2.2d",i);
      sprintf(std,"hEvtX%2.2d 100,0.,1000.,1000,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdef7[i].title,
			&hdef7[i].nxbin, &hdef7[i].xmin, &hdef7[i].xmax,
			&hdef7[i].nybin, &hdef7[i].ymin, &hdef7[i].ymax);		   }
   hRes    = new TH2F* [kNh];
   hPhSum  = new TH2F* [kNh];
   hPhPk   = new TH2F* [kNh];
   hPed    = new TH2F* [kNh];
   hEff    = new TH1E* [kNh];
   hTwk    = new TH2F* [kNh];
   hSgt2   = new TH2F* [kNh];
   hEff2tk = new TH1E* [kNh];
   hSepRes = new TH2F* [kNh];
   hTC2Sep = new TH2F* [kNh];
   hSumw5  = new TH2F* [kNh];
   hHodX   = new TH2F* [kNh];
   hEvtX   = new TH2F* [kNh];
   hXdif   = new TH2F* [kNh];
}

//___________________________________________________________________________
U4BabyAnalysis::~U4BabyAnalysis()
{
   delete [] hdefr;
   delete [] hdefs;
   delete [] hdefp;
   delete [] hdefz;
   delete [] hdefe;
   delete [] hdefw;
   delete [] hdefg;
   delete [] hdeft;
   delete [] hdefu;
   delete [] hdefa;
   delete [] hdefn;
   delete [] hdefh;
   delete [] hdef7;
   Int_t i;
   for (i = 0; i < kNh; i++) {
   	delete hRes   [i];
   	delete hPhSum [i];
   	delete hPhPk  [i];
   	delete hPed   [i];
	delete hEff   [i];
	delete hTwk   [i];
	delete hSgt2  [i];
	delete hEff2tk[i];
	delete hSepRes[i];
	delete hTC2Sep[i];
        delete hSumw5 [i];
        delete hHodX  [i];
        delete hEvtX  [i];
        delete hXdif  [i];
   }
   delete [] hRes;
   delete [] hPhSum;
   delete [] hPhPk;
   delete [] hPed;
   delete [] hEff;
   delete [] hTwk;
   delete [] hSgt2;
   delete [] hEff2tk;
   delete [] hSepRes;
   delete [] hTC2Sep;
   delete [] hSumw5;
   delete [] hHodX;
   delete [] hEvtX;
   delete [] hXdif;
}

//___________________________________________________________________________
Bool_t U4BabyAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < kNh; i++) {
      hRes[i] = new TH2F(hdefr[i].title,hdefr[i].title,
      			hdefr[i].nxbin,hdefr[i].xmin,hdefr[i].xmax,
      			hdefr[i].nybin,hdefr[i].ymin,hdefr[i].ymax);
      std::cout << i << " "      << hdefr[i].title 
      		<< " nx "   << hdefr[i].nxbin
      		<< " xmin " << hdefr[i].xmin
      		<< " xmax " << hdefr[i].xmax
      		<< " ny "   << hdefr[i].nybin
      		<< " ymin " << hdefr[i].ymin
      		<< " ymax " << hdefr[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hPhSum[i] = new TH2F(hdefs[i].title,hdefs[i].title,
      			hdefs[i].nxbin,hdefs[i].xmin,hdefs[i].xmax,
      			hdefs[i].nybin,hdefs[i].ymin,hdefs[i].ymax);
      std::cout << i << " "      << hdefs[i].title 
      		<< " nx "   << hdefs[i].nxbin
      		<< " xmin " << hdefs[i].xmin
      		<< " xmax " << hdefs[i].xmax
      		<< " ny "   << hdefs[i].nybin
      		<< " ymin " << hdefs[i].ymin
      		<< " ymax " << hdefs[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hPhPk[i] = new TH2F(hdefp[i].title,hdefp[i].title,
      			hdefp[i].nxbin,hdefp[i].xmin,hdefp[i].xmax,
      			hdefp[i].nybin,hdefp[i].ymin,hdefp[i].ymax);
      std::cout << i << " "      << hdefp[i].title 
      		<< " nx "   << hdefp[i].nxbin
      		<< " xmin " << hdefp[i].xmin
      		<< " xmax " << hdefp[i].xmax
      		<< " ny "   << hdefp[i].nybin
      		<< " ymin " << hdefp[i].ymin
      		<< " ymax " << hdefp[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hPed[i] = new TH2F(hdefz[i].title,hdefz[i].title,
      			hdefz[i].nxbin,hdefz[i].xmin,hdefz[i].xmax,
      			hdefz[i].nybin,hdefz[i].ymin,hdefz[i].ymax);
      std::cout << i << " "      << hdefz[i].title 
      		<< " nx "   << hdefz[i].nxbin
      		<< " xmin " << hdefz[i].xmin
      		<< " xmax " << hdefz[i].xmax
      		<< " ny "   << hdefz[i].nybin
      		<< " ymin " << hdefz[i].ymin
      		<< " ymax " << hdefz[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hEff[i] = new TH1E(hdefe[i].title,hdefe[i].title,
      			hdefe[i].nxbin,hdefe[i].xmin,hdefe[i].xmax);
      std::cout << i << " "      << hdefe[i].title 
      		<< " nx "   << hdefe[i].nxbin
      		<< " xmin " << hdefe[i].xmin
      		<< " xmax " << hdefe[i].xmax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hTwk[i] = new TH2F(hdefw[i].title,hdefw[i].title,
      			hdefw[i].nxbin,hdefw[i].xmin,hdefw[i].xmax,
      			hdefw[i].nybin,hdefw[i].ymin,hdefw[i].ymax);
      std::cout << i << " "      << hdefw[i].title 
      		<< " nx "   << hdefw[i].nxbin
      		<< " xmin " << hdefw[i].xmin
      		<< " xmax " << hdefw[i].xmax
      		<< " ny "   << hdefw[i].nybin
      		<< " ymin " << hdefw[i].ymin
      		<< " ymax " << hdefw[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hSgt2[i] = new TH2F(hdefg[i].title,hdefg[i].title,
      			hdefg[i].nxbin,hdefg[i].xmin,hdefg[i].xmax,
      			hdefg[i].nybin,hdefg[i].ymin,hdefg[i].ymax);
      std::cout << i << " "      << hdefg[i].title 
      		<< " nx "   << hdefg[i].nxbin
      		<< " xmin " << hdefg[i].xmin
      		<< " xmax " << hdefg[i].xmax
      		<< " ny "   << hdefg[i].nybin
      		<< " ymin " << hdefg[i].ymin
      		<< " ymax " << hdefg[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hEff2tk[i] = new TH1E(hdeft[i].title,hdeft[i].title,
      			hdeft[i].nxbin,hdeft[i].xmin,hdeft[i].xmax);
      std::cout << i << " "      << hdeft[i].title 
      		<< " nx "   << hdeft[i].nxbin
      		<< " xmin " << hdeft[i].xmin
      		<< " xmax " << hdeft[i].xmax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hSepRes[i] = new TH2F(hdefu[i].title,hdefu[i].title,
      			hdefu[i].nxbin,hdefu[i].xmin,hdefu[i].xmax,
      			hdefu[i].nybin,hdefu[i].ymin,hdefu[i].ymax);
      std::cout << i << " "      << hdefu[i].title 
      		<< " nx "   << hdefu[i].nxbin
      		<< " xmin " << hdefu[i].xmin
      		<< " xmax " << hdefu[i].xmax
      		<< " ny "   << hdefu[i].nybin
      		<< " ymin " << hdefu[i].ymin
      		<< " ymax " << hdefu[i].ymax
      		<< std::endl;
   }
   for (i = 0; i < kNh; i++) {
      hTC2Sep[i] = new TH2F(hdefa[i].title,hdefa[i].title,
      			hdefa[i].nxbin,hdefa[i].xmin,hdefa[i].xmax,
      			hdefa[i].nybin,hdefa[i].ymin,hdefa[i].ymax);
      std::cout << i << " "      << hdefa[i].title 
      		<< " nx "   << hdefa[i].nxbin
      		<< " xmin " << hdefa[i].xmin
      		<< " xmax " << hdefa[i].xmax
      		<< " ny "   << hdefa[i].nybin
      		<< " ymin " << hdefa[i].ymin
      		<< " ymax " << hdefa[i].ymax
      		<< std::endl;
   }

   for (i = 0; i < kNh; i++) {
      hSumw5[i] = new TH2F(hdefn[i].title,hdefn[i].title,
      			hdefn[i].nxbin,hdefn[i].xmin,hdefn[i].xmax,
      			hdefn[i].nybin,hdefn[i].ymin,hdefn[i].ymax);
      std::cout << i << " "      << hdefn[i].title 
      		<< " nx "   << hdefn[i].nxbin
      		<< " xmin " << hdefn[i].xmin
      		<< " xmax " << hdefn[i].xmax
      		<< " ny "   << hdefn[i].nybin
      		<< " ymin " << hdefn[i].ymin
      		<< " ymax " << hdefn[i].ymax
      		<< std::endl;
   }
 
   for (i = 0; i < kNh; i++) {
      hHodX[i] = new TH2F(hdefh[i].title,hdefh[i].title,
      			hdefh[i].nxbin,hdefh[i].xmin,hdefh[i].xmax,
      			hdefh[i].nybin,hdefh[i].ymin,hdefh[i].ymax);
      std::cout << i << " "      << hdefh[i].title 
      		<< " nx "   << hdefh[i].nxbin
      		<< " xmin " << hdefh[i].xmin
      		<< " xmax " << hdefh[i].xmax
      		<< " ny "   << hdefh[i].nybin
      		<< " ymin " << hdefh[i].ymin
      		<< " ymax " << hdefh[i].ymax
      		<< std::endl;
   }
   
   for (i = 0; i < kNh; i++) {
      hEvtX[i] = new TH2F(hdef7[i].title,hdef7[i].title,
      			hdef7[i].nxbin,hdef7[i].xmin,hdef7[i].xmax,
      			hdef7[i].nybin,hdef7[i].ymin,hdef7[i].ymax);
      std::cout << i << " "      << hdef7[i].title 
      		<< " nx "   << hdef7[i].nxbin
      		<< " xmin " << hdef7[i].xmin
      		<< " xmax " << hdef7[i].xmax
      		<< " ny "   << hdef7[i].nybin
      		<< " ymin " << hdef7[i].ymin
      		<< " ymax " << hdef7[i].ymax
      		<< std::endl;
   }

   hXdif[0] = new TH2F("hEvtXdiflo","hEvtXdiflo",1000,0.,1000.,100,-.2,.6);
   hXdif[1] = new TH2F("hEvtXdifhi","hEvtXdifhi",1000,0.,1000.,100,-.2,.6);
   hXdif[2] = new TH2F("hHodXdif"  ,"hHodXdif"  ,100,0.,10000.,100,-.2,.6);
   hXdif[3] = new TH2F("hCurXdif"  ,"hCurXdif"  ,100,0.,1.e-2,100,-.2,.6);
   
   hcp = new TCanvas("hcp","Canvas p",20, 20, 200*kZoneX, 200*kZoneY);
   hcp->Divide(kZoneX,kZoneY);
   hcs = new TCanvas("hcs","Canvas s",20, 20, 200*kZoneX, 200*kZoneY);
   hcs->Divide(kZoneX,kZoneY);
   hcz = new TCanvas("hcz","Canvas z",20, 20, 200*kZoneX, 200*kZoneY);
   hcz->Divide(kZoneX,kZoneY);
   hcr = new TCanvas("hcr","Canvas r",20, 20, 200*kZoneX, 200*kZoneY);
   hcr->Divide(kZoneX,kZoneY);
   hce = new TCanvas("hce","Canvas e",20, 20, 200*kZoneX, 200*kZoneY);
   hce->Divide(kZoneX,kZoneY);
   hcw = new TCanvas("hcw","Canvas w",20, 20, 200*kZoneX, 200*kZoneY);
   hcw->Divide(kZoneX,kZoneY);
   hcg = new TCanvas("hcg","Canvas g",20, 20, 200*kZoneX, 200*kZoneY);
   hcg->Divide(kZoneX,kZoneY);
   hct = new TCanvas("hct","Canvas t",20, 20, 200*kZoneX, 200*kZoneY);
   hct->Divide(kZoneX,kZoneY);
   hcu = new TCanvas("hcu","Canvas u",20, 20, 200*kZoneX, 200*kZoneY);
   hcu->Divide(kZoneX,kZoneY);
   hca = new TCanvas("hca","Canvas a",20, 20, 200*kZoneX, 200*kZoneY);
   hca->Divide(kZoneX,kZoneY);
   hcn = new TCanvas("hca","Canvas a",20, 20, 200*kZoneX, 200*kZoneY);
   hcn->Divide(kZoneX,kZoneY);
   hch = new TCanvas("hca","Canvas a",20, 20, 200*kZoneX, 200*kZoneY);
   hch->Divide(kZoneX,kZoneY);
   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t U4BabyAnalysis::Process(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   Int_t evtno = gJSF->GetEventNumber();
   U4BabyTrackMaker *tm = (U4BabyTrackMaker*)gJSF->FindModule("U4BabyTrackMaker");
   U4BabyTrackMakerBuf   *tb = (U4BabyTrackMakerBuf *)tm->EventBuf();
   TObjArray     *outputbufp = tb->GetOutputBufPtr();

   U4CDCTrack::SetSortBy("Time");
   outputbufp->UnSort();
   outputbufp->Sort();

   U4BabyTrack *tp;
   U4BabyTrack *tplst = 0;
   TIter nexttrack(outputbufp);
   TObjArray tracks[2];
   while ((tp = (U4BabyTrack *)nexttrack())) {
#if 0
      U4CDCHit *htp = ((U4CDCTrackHit *)tp->At(0))->GetHitPtr();
#endif
      U4CDCVectorHit *vhtp = (U4CDCVectorHit *)tp->At(0);
#if 1 
      std::cerr << " vhtp = " << (void *)vhtp 
	        << "  tp = " << (void *)tp << std::endl;
#endif
      Int_t thislayer = vhtp->GetLayerNumber();
      Int_t thiscell  = vhtp->GetCellNumber();
      if (thiscell != 0) continue;		// skip unequipped cell
      tracks[thislayer].Add(tp);
   }
   
   Int_t itk1st  = fTk1st;			// track No. range (1st)
   Int_t itklst  = fTklast;			//		   (lst)
   Double_t adctc1, adctc2, adcche, adctof1, adctof2, tof;
   GetCounterInfo(adctc1,adctc2,adcche,adctof1,adctof2,tof);

   Int_t sclhodx[5], sclhody[3];
   Int_t scltc1, scltc2, sclche, scltof1, scltof2;
   Int_t sclpi, sclp, scle, sclnopid;
   GetScalerInfo(sclhodx,sclhody,scltc1,scltc2,sclche,scltof1,scltof2,
   		 sclpi,sclp,scle,sclnopid);
   Double_t gpicurr;
   GetGPIBInfo(gpicurr);
   
   Int_t sclhodsum = 0;
   for (Int_t ix = 0; ix < 5; ix++) sclhodsum += sclhodx[ix];
   for (Int_t ix = 0; ix < 3; ix++) sclhodsum += sclhody[ix];

   for (Int_t i = 0; i < U4BabyAnalysis::kNh; i++) fPhSum[i] = 0;
   Double_t xat7sv[] = { -9999., 9999. };

   for (Int_t ilyr = 0; ilyr < 2; ilyr++) {   
      Int_t ntracks = tracks[ilyr].GetEntries();
      itklst = TMath::Min(fTklast,ntracks-1);
      for (Int_t itk = itk1st; itk <= itklst; itk++) {
         tp = (U4BabyTrack *)tracks[ilyr].At(itk);
         if (!tplst && itk > 0) tplst = (U4BabyTrack *)tracks[ilyr].At(itk-1);
         Int_t nwires = tp->GetEntries();
         if (nwires < fNwires) continue;	// require fNwires wires
	 U4CDCVectorHit *vhtp;
         U4CDCTrackHit  *hitp;
         U4CDCHit       *htp;
         U4BabyTrack     trk(*tp);
         TIter next(&trk);
         Stat_t wat[U4BabyAnalysis::kNh];
         
         Double_t xat7 = trk.Xat(7.); // x at 7cm (shielding wires) 
         if (TMath::Abs(xat7sv[ilyr]) > 999. && xat7 > xX7lo && xat7 < xX7hi) {
         	xat7sv[ilyr] = xat7;
#ifdef __DEBUG__
		std::cerr	<< " xX7lo = " << xX7lo << " xX7hi = " << xX7hi << std::endl;
	 	std::cerr	<< " ilyr = " << ilyr
			<< " xat7 = " << xat7
			<< std::endl;
#endif
         }  
         if (sclhodsum < xSclHodSum) {
            	hEvtX[2*ilyr+0]->Fill(evtno+.1,xat7,1.); // between spills
         } else {
            	hEvtX[2*ilyr+1]->Fill(evtno+.1,xat7,1.); // during spill
         }
       
         Int_t nhit = 0;
         Double_t q5 = 0;
         Double_t xsv[100];

      	 next.Reset();

#if 0
         while ((hitp = (U4CDCTrackHit *)next())) {
#else
	 while ((vhtp = (U4CDCVectorHit *)next())) {
	    TIter nexttht(vhtp);
	    while ((hitp = (U4CDCTrackHit *)nexttht())) {
#endif
            	htp = hitp->GetHitPtr();
            	Int_t layer = htp->GetLayerNumber();
            	Int_t cell  = htp->GetCellNumber();
            	Int_t wire  = htp->GetWireNumber();
		Int_t a     = 5*layer + wire;   // works only for baby
            	if (cell == 1) a = 10 + wire;	// works only for baby
            	if ( a < 0 || a >= U4BabyAnalysis::kNh ) continue;
            	Double_t y    = htp->Ywire();
            	Double_t x    = trk.Xat(y) - htp->Xwire();
            
            	hHodX[a]->Fill(sclhodsum+.1,x,1.); // hod counts vs drift length
            
	    	if (layer == 1 && cell == 0) x *= -1.;
            	Double_t ph   = ((U4CDCCluster *)hitp->GetHitPtr()->GetClusterPtr())->GetSum();
            	xsv[nhit] = x;
		
               		if (ph < xPh){
                   		q5 += ph;
                   		nhit++;
			}
            
           	if (nhit == 5) hSumw5[0]->Fill(xsv[2],q5/nhit,1.0); //ch5 average
	   	fPhSum[a] = ph;
	    }
         }

         for (Int_t i = 0; i < U4BabyAnalysis::kNh; i++) wat[i] = 0;
         Int_t tolook;
         for (tolook = 5*ilyr; tolook < 5*(ilyr+1); tolook++) {
      	    next.Reset();
      	    Int_t nref = 0;
      	    U4CDCTrackHit *actualp = 0;
#if 0
            while ((hitp = (U4CDCTrackHit *)next())) {
#else
	    while ((vhtp = (U4CDCVectorHit *)next())) {
	       TIter nexttht(vhtp);
	       while ((hitp = (U4CDCTrackHit *)nexttht())) {
#endif
                  htp = hitp->GetHitPtr();
                  Int_t layer = htp->GetLayerNumber();
                  Int_t cell  = htp->GetCellNumber();
                  Int_t wire  = htp->GetWireNumber();
                  Int_t a     = 5*layer + wire;	// works only for baby
                  if (cell == 1) a = 10 + wire;	// works only for baby
                  if ( a < 0 || a >= U4BabyAnalysis::kNh ) continue;
                  if (a == tolook) {
                     actualp = hitp;
                  } else { 
                     nref++;
                  }
	       }
            }
            if (nref < 4) continue;
            if (actualp && fMode == 0) {
	       actualp->Lock();			// lock this hit and
               trk.Update();			// exclude it from fit
               if (trk.GetChi2() > xChi2) {
                  actualp->Unlock();
                  break;
               }
            }
            if (actualp) {
               htp = actualp->GetHitPtr();
               Int_t layer = htp->GetLayerNumber();
               Int_t cell  = htp->GetCellNumber();
               Int_t wire  = htp->GetWireNumber();
               Int_t a     = 5*layer + wire;	// works only for baby
               if (cell == 1) a = 10 + wire;	// works only for baby
               Double_t y    = htp->Ywire();
               Double_t x    = trk.Xat(y) - htp->Xwire();
	       if (layer == 1 && cell == 0) x *= -1.;
               Double_t dx   = actualp->GetResidual();
               Double_t ph   = ((U4CDCCluster *)actualp->GetHitPtr()->GetClusterPtr())->GetSum();
               Double_t php  = ((U4CDCCluster *)actualp->GetHitPtr()->GetClusterPtr())->GetPeak();
               Double_t ped  = ((U4CDCCluster *)actualp->GetHitPtr()->GetClusterPtr())->GetPedestal();
               Double_t sgx2 = trk.SGX2at(htp->Ywire());
	       // std::cerr << "sgx2 = " << sgx2 << std::endl;
      	       hRes  [a]->Fill(x,dx,1.0);
      	       hPhSum[a]->Fill(x,ph,1.0);
      	       hPhPk [a]->Fill(x,php,1.0);
      	       hPed  [a]->Fill(x,ped,1.0);
               hTwk  [a]->Fill(php,dx,1.0);
               hSgt2 [a]->Fill(x,sgx2,1.0);
                  
               if (tplst) {
                  Double_t sep = tp->Xat(y) - tplst->Xat(y);
	          if (layer == 1 && cell == 0) sep *= -1.;
                  hSepRes[a]->Fill(sep,dx,1.0);
                  hTC2Sep[a]->Fill(adctc2,sep,1.0);
               }
               if (TMath::Abs(dx) < xNsigmas*TMath::Sqrt(sgx2)) wat[a] = 1.;
               if (fMode == 0) {
	          actualp->Unlock();		// unlock this hit
	       }
            }

      	    Double_t x, y;
      	    Int_t layer, cell, wire;
      	    U4CDCHitWire::ChannelToWire(tolook,layer,cell,wire,x,y);
      	    hEff[tolook]->Fill(tp->Xat(y),wat[tolook]);
      	    if (tplst) {
      	       Double_t sep = tp->Xat(y) - tplst->Xat(y);
	       if (layer == 1 && cell == 0) sep *= -1.;
      	       hEff2tk[tolook]->Fill(sep,wat[tolook]);
      	    }
         }
         tplst = tp;
      }
      if (!(gROOT->IsBatch()) && ev%200 == 1) Draw();
   }
   
   Double_t xdiff = xat7sv[0] - xat7sv[1];
   if (sclhodsum < xSclHodSum) {
      hXdif[0]->Fill(evtno+.1,xdiff,1.);
   } else {
      hXdif[1]->Fill(evtno+.1,xdiff,1.);
   }
   hXdif[2]->Fill(sclhodsum+.1,xdiff,1.);
   hXdif[3]->Fill(gpicurr,xdiff,1.);
#ifdef __DEBUG__
std::cerr 	<< " EvtNo = " << evtno << " HodSum = " << sclhodsum
	<< " GPIB Current = " << gpicurr << " Xdiff = " << xdiff << std::endl;
#endif
   last->cd();
   return kTRUE;
}

//___________________________________________________________________________
void U4BabyAnalysis::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < U4BabyAnalysis::kNh; i++) {
      hcp->cd(i+1); hPhPk  [i]->Draw(); hcp->Update();
      hcs->cd(i+1); hPhSum [i]->Draw(); hcs->Update();
      hcz->cd(i+1); hPed   [i]->Draw(); hcz->Update();
      hcr->cd(i+1); hRes   [i]->Draw(); hcr->Update();
      hce->cd(i+1); hEff   [i]->Draw();	hce->Update();
      hcw->cd(i+1); hTwk   [i]->Draw();	hcw->Update();
      hcg->cd(i+1); hSgt2  [i]->Draw();	hcg->Update();
      hct->cd(i+1); hEff2tk[i]->Draw();	hct->Update();
      hcu->cd(i+1); hSepRes[i]->Draw();	hcu->Update();
      hca->cd(i+1); hTC2Sep[i]->Draw();	hca->Update();
   }
      hcn->cd(1); hSumw5[0]->Draw(); 	hcn->Update();
   last->cd();
}

//---------------------------------------------------------------------------
void U4BabyAnalysis::GetCounterInfo(Double_t &adctc1, Double_t &adctc2, 
                                 Double_t &adcche, Double_t &adctof1, 
                                 Double_t &adctof2, Double_t &tof)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
   U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
   U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
   Int_t fmtid = br->GetHeader()->GetFormatID();

   adctc1 = adctc2 = adcche = adctof1 = adctof2 = tof = 0;
   if (fmtid != 2) return;

   Int_t           i1st  = evt->GetNonFADCIndex();
   Int_t           ndata = evt->GetNADCData();
   TClonesArray   *data  = evt->GetADCData();
   for (Int_t i = i1st; i < ndata; i++) {
      U4CDCRawADCDatum *datum = (U4CDCRawADCDatum *)data->UncheckedAt(i);
      Int_t st = datum->GetStationNo();
      Int_t a;
      Int_t id;
      Int_t cr = 0;               // We have only 1 crate now.
      Int_t ch = datum->GetChannelNo();
      br->EChannelToDChannel(cr,st,ch,id,a);
      if (id == 0 || id == 2) continue;
      Double_t adc = -999999;
      Double_t tdc = -999999;
      switch (id) {
      	case 1: // ADC
      		adc  = datum->GetADC();
      		switch (a) {
           	  case 0:
            		adctof1 = adc;
                	break;
           	  case 1:
            		adctof2 = adc;
                	break;
           	  case 2:
            		adctc1  = adc;
                	break;
           	  case 3:
            		adctc2  = adc;
                	break;
           	  case 4:
            		adcche  = adc;
                	break;
           	}
          	break;
      	case 3: // TOF
       		tdc  = datum->GetTime();
      		switch (a) {
           	  case 0:
            		tof = adc;
                	break;
           	}
           default:
                break;
      }
   } 
   last->cd();
#if __DEBUG__
std::cerr	<< " ------------------------------------- " << std::endl
	<< " Counter Information for Run " << gJSF->GetRunNumber()
	<< " Event " << gJSF->GetEventNumber() << std::endl
	<< " ------------------------------------- " << std::endl;
std::cerr 	<< " adctc1 = " << adctc1 << " adctc2 = " << adctc2
	<< " adcche = " << adcche
	<< " adctof1 = " << adctof1 << " adctof2 = " << adctof2 
	<< " tof = " << tof
	<< std::endl;
#endif
}

//---------------------------------------------------------------------------
void U4BabyAnalysis::GetScalerInfo(Int_t sclhodx[], Int_t sclhody[],
				Int_t &scltc1, Int_t &scltc2, Int_t &sclche, 
				Int_t &scltof1, Int_t &scltof2,
				Int_t &sclpi, Int_t &sclp, Int_t &scle,
				Int_t &sclnopid)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
   U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
   U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
   Int_t fmtid = br->GetHeader()->GetFormatID();
   
   for (Int_t j = 0; j < 5; j++) { sclhodx[j] = 0; }
   for (Int_t j = 0; j < 3; j++) { sclhody[j] = 0; }
   scltc1 = scltc2 = sclche = scltof1 = scltof2 = 0;
   sclpi = sclp = scle = sclnopid = 0;
   if (fmtid != 2) return;

   Int_t           ndata = evt->GetNScalerData();
   TClonesArray   *data  = evt->GetScalerData();
   for (Int_t i = 0; i < ndata; i++) {
      U4CDCRawScalerDatum *datum = (U4CDCRawScalerDatum *)data->UncheckedAt(i);
      Int_t st = datum->GetStationNo();
      Int_t a;
      Int_t id;
      Int_t cr = 0;               // We have only 1 crate now.
      Int_t ch = datum->GetChannelNo();
      br->EChannelToDChannel(cr,st,ch,id,a);
      if (id != 2) continue;
      Int_t value = datum->GetScaler();
      switch (a) {
      	case 0:
      	case 1:
      	case 2:
      	case 3:
      	case 4:
      		sclhodx[a] = value;
      		break;
      	case 6:
      	case 7:
      	case 5:
      		sclhody[a-5] = value;
      		break;
      	case 12:
      		scltof1 = value;
      		break;
      	case 13:
      		scltof2 = value;
      		break;
      	case 14:
      		scltc1 = value;
      		break;
      	case 15:
      		scltc2 = value;
      		break;
      	case 16:
      		sclche = value;
      		break;
      	case 19:
      		sclpi = value;
      		break;
      	case 20:
      		sclp  = value;
      		break;
      	case 21:
      		scle  = value;
      		break;
      	case 22:
      		sclnopid = value;
      		break;
      	default:
                break;
      }
   } 
   last->cd();
#if __DEBUG__
std::cerr	<< " ------------------------------------- " << std::endl
	<< " Scaler Values for Run " << gJSF->GetRunNumber() 
	<< " Event " << gJSF->GetEventNumber() << std::endl
	<< " ------------------------------------- " << std::endl;
std::cerr 	<< " sclhodx[] = ";
for (Int_t j = 0; j < 5; j++) { std::cerr << sclhodx[j] << " "; }
std::cerr	<< std::endl;
std::cerr 	<< " sclhody[] = ";
for (Int_t j = 0; j < 3; j++) { std::cerr << sclhody[j] << " "; }
std::cerr	<< std::endl;
std::cerr	<< " scltc1 = " << scltc1 << " scltc2 " << scltc2
	<< " sclche = " << sclche
	<< " scltof1 = " << scltof1 << " scltof2 = " << scltof2
	<< std::endl
	<< " sclpi = " << sclpi << " sclp = " << sclp
	<< " scle = " << scle << " sclnopid = " << sclnopid
	<< std::endl;
#endif
}

//---------------------------------------------------------------------------
void U4BabyAnalysis::GetGPIBInfo(Double_t &gpicurr)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
   U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
   U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
   Int_t fmtid = br->GetHeader()->GetFormatID();
   
   gpicurr = 0.;
   if (fmtid != 2) return;

   Int_t           ndata = evt->GetNGPIBData();
   TClonesArray   *data  = evt->GetGPIBData();
   for (Int_t i = 0; i < ndata; i++) {
      U4CDCRawGPIBDatum *datum = (U4CDCRawGPIBDatum *)data->UncheckedAt(i);
      Int_t st = datum->GetStationNo();
      Int_t a;
      Int_t id;
      Int_t cr = 0;               // We have only 1 crate now.
      Int_t ch = datum->GetChannelNo();
      br->EChannelToDChannel(cr,st,ch,id,a);
      if (id != 2) continue;
      Double_t value = datum->GetGPIB();
      switch (a) {
      	case 100:
      		gpicurr = value;
      		break;
      	default:
                break;
      }
   } 
   last->cd();
#if __DEBUG__
std::cerr	<< " ------------------------------------- " << std::endl
	<< " GPIB Values for Run " << gJSF->GetRunNumber() 
	<< " Event " << gJSF->GetEventNumber() << std::endl
	<< " ------------------------------------- " << std::endl;
std::cerr 	<< " gpib current = " << gpicurr << std::endl;
#endif
}
