#ifndef __U4CDCCLUSTERBUF__
#define __U4CDCCLUSTERBUF__

//***********************************************************************
//* ------------------------------
//* Cluster Buffer
//* ------------------------------
//***********************************************************************

#include <iostream>
#include <iomanip>
#include "TClass.h"
#include "TObjArray.h"

#include "JSFSteer.h"
#include "JSFModule.h"
#include "U4CDCRawData.h"

class U4CDCClusterMaker;

class U4CDCClusterBuf : public JSFEventBuf {
friend class U4CDCClusterMaker;
public:
  U4CDCClusterBuf(U4CDCFADCCalibration *calib=0,
                const char *name="U4CDCClusterBuf",
                const char *title="U4CDC cluster data", 
                U4CDCClusterMaker *mod=0);
  U4CDCClusterBuf(U4CDCClusterMaker *mod, U4CDCFADCCalibration *calib=0,
                const char *name="U4CDCClusterBuf",
                const char *title="U4CDC cluster data");
  virtual ~U4CDCClusterBuf();

  void          SetADCData(Len_t nadcdata, TClonesArray *adcdata);
  void          SetCalibrationData(U4CDCFADCCalibration *calib);

  Bool_t        IsInitialized() const;  // inquires if input data set
  Int_t         GetNclusters();         // returns No. clusters
  TObjArray    *GetClusters();          // returns array of clusters
  void          UseVarPed();            // use variable pedestal
  void          UseConstPed();          // use constant pedestal

  void          DebugPrint(const Char_t *opt = "Brief");

private:
  void          FindClusters();         // finds clusters

private:
  Bool_t              fDone;            // (1,0) = (clustering done, not yet)
  Bool_t              fUseVarPed;       // (1,0) = (var ped, const ped)
  Len_t               fNADCData;        // No. ADC data units
  TClonesArray       *fADCData;         // Array of ADC Data
  TObjArray           fGlobalClusters;  // array of global clusters
  TObjArray           fClusters;        // array of clusters
  U4CDCFADCCalibration *fCalib;           // pointer to calibration data object

  ClassDef(U4CDCClusterBuf, 1) // U4CDC cluster data buffer
};

#endif
