#ifndef __U4CDCFADCCALIBRATION__
#define __U4CDCFADCCALIBRATION__

//*************************************************************************
//* -----------------------------
//* FADC Calibration Data
//* -----------------------------
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "TClass.h"
#include "TObjArray.h"

#include "JSFSteer.h"
#include "JSFModule.h"
#include "U4CDCRawData.h"

class U4CDCFADCCalibration : public TObjArray {
private:
  class CalibDatum : public TObject {
     public:
        CalibDatum(Int_t last = 9999999,
                   Double_t *ped  = 0, Double_t *gain   = 0, 
                   Double_t *tped = 0, Double_t *tscale = 0) : fLastRun(last)
        {
           Int_t n = U4CDCRawADCDatum::GetNchMax();
           fPedestal     = new Double_t [n];
           fGain         = new Double_t [n];
           fTimePedestal = new Double_t [n];
           fTimeScale    = new Double_t [n];
           Int_t i;
           if (!ped)    for (i=0; i<n; i++) fPedestal    [i] = 200.;
           else         for (i=0; i<n; i++) fPedestal    [i] = ped[i];
           if (!gain)   for (i=0; i<n; i++) fGain        [i] =   1.;
           else         for (i=0; i<n; i++) fGain        [i] = gain[i];
           if (!tped)   for (i=0; i<n; i++) fTimePedestal[i] =   0.;
           else         for (i=0; i<n; i++) fTimePedestal[i] = tped[i];
           if (!tscale) for (i=0; i<n; i++) fTimeScale   [i] =   2.;
           else         for (i=0; i<n; i++) fTimeScale   [i] = tscale[i];
        }
        ~CalibDatum() {
           if (fPedestal)     delete [] fPedestal;
           if (fGain)         delete [] fGain;
           if (fTimePedestal) delete [] fTimePedestal;
           if (fTimeScale)    delete [] fTimeScale;
        }
        Int_t    GetLastRun() { return fLastRun; }      
        Double_t GetPedestal    (Int_t n, Int_t a)
        {
                return fPedestal[ChannelToWire(n,a)];
        }
        Double_t GetGain        (Int_t n, Int_t a)
        {
                return fGain[ChannelToWire(n,a)];
        }
        Double_t GetTimePedestal(Int_t n, Int_t a)
        {
                return fTimePedestal[ChannelToWire(n,a)];
        }
        Double_t GetTimeScale   (Int_t n, Int_t a)
        {
                return fTimeScale[ChannelToWire(n,a)];
        }
     private:

        Int_t ChannelToWire(Int_t n, Int_t a)
        {
           U4CDCRawData     *rd = (U4CDCRawData *)gJSF->FindModule("U4CDCRawData");
           U4CDCBeginRunBuf *br = rd->GetBeginRunBuf();
           Int_t id, w; 
           Int_t cr = 0; // Only one crate at the moment.
           br->EChannelToDChannel(cr,n,a,id,w);
           return w;
        }

     private:
        Int_t    fLastRun;              //! Last Run Number
        Double_t *fPedestal;            //! Pedestal
        Double_t *fGain;                //! Gain
        Double_t *fTimePedestal;        //! Time Pedestal
        Double_t *fTimeScale;           //! Time Scale
  };
  
public:
  U4CDCFADCCalibration(Double_t *ped  = 0, Double_t *gain   = 0, 
                     Double_t *tped = 0, Double_t *tscale = 0,
                     Int_t last = 9999999);
  U4CDCFADCCalibration(Char_t * calibfile);
  virtual ~U4CDCFADCCalibration();
  
  Double_t GetPedestal    (Int_t n, Int_t a); 
  Double_t GetGain        (Int_t n, Int_t a);
  Double_t GetTimePedestal(Int_t n, Int_t a);
  Double_t GetTimeScale   (Int_t n, Int_t a);

  void Update();
  
private:
  CalibDatum *fCurr;                    //! pointer to current calib datum

  ClassDef(U4CDCFADCCalibration, 1)       // U4CDC FADC Calibration
};

#endif
