#ifndef __U4CDCKALDETECTOR__
#define __U4CDCKALDETECTOR__

#include <iostream>

#include "TVector3.h"

#include "U4VKalDetector.h"
#include "U4CDCMeasLayer.h"

using namespace std;

class U4CDCKalDetector : public U4VKalDetector {
public:
   U4CDCKalDetector();

   ~U4CDCKalDetector() {}

   Double_t CalcRadLength(const TVMeasLayer &from,
                          const TVMeasLayer &to) const;
   Double_t CalcSigmaMS0 (const TVMeasLayer &from,
                          const TVMeasLayer &to,
                                Double_t     pathlen) const;

   static U4CDCKalDetector *GetInstance();

   static void   SetAxialOnly(Bool_t ao = kTRUE) { fgIsAxialOnly = ao;   }
   static Bool_t IsAxialOnly()                   { return fgIsAxialOnly; }
          Double_t GetAvRadius() const; 

private:
   Double_t fTwistedAngle[10];               // twist angle
   Double_t fLayerPhiOffset[10];             // phi offset for layer
   Double_t fSuperLayerPhiOffset[4];         // phi offset for super layer
   Double_t fLayerR[10];                     // layer radius
   Int_t    fNcellsParLayer[4];              // No. cells per layer for each sup.layer
   Int_t    fSuperLayerInnerGap;             // inner gap between sup. layer
   Int_t    fSuperLayerOuterGap;             // outer gap between sup. layer
   Int_t    fAxialToStereoGap;               // axial to stereo gap
   Int_t    fStereoToStereoGap;              // stereo to stereo gap
   Int_t    fCDCInnerR;                      // cdc inner radius
   Int_t    fCDCOuterR;                      // cdc outer radius
   Int_t    fCDCHalfZ;                       // cdc half Z length
   Int_t    fNsuperLayers;                   // No. super layers
   Int_t    fNlayers;                        // No. layers
   Int_t    fNwires;                         // No. wires
   Double_t fCellThick;                      // cell thickness
   Double_t fThick;                          // dirift cell thickness
   Double_t fDummyThick;                     // dummy drift cell thickness
   Double_t fStereoToAxialGap;               // stereo to axial gap
   Double_t fSupportTubHalfThick;            // support tube half thickness

   static U4CDCKalDetector *fgInstance;      //!
   static Bool_t            fgIsAxialOnly;   //!

   ClassDef(U4CDCKalDetector,1)           // Sample hit class
};

#endif
