#ifndef __U4CDCHIT__
#define __U4CDCHIT__

//*************************************************************************
//* ---------------------
//* U4CDCHit Class
//* ---------------------
//* (Description)
//*	Class for a single CDC hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCHit		: TObject containing single hit data
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "TAttLockable.h"
#include "U4CDCCluster.h"
#include "U4CDCHitWire.h"

class U4CDCHitWire;
class U4CDCHit : public TObject, public TAttLockable {
friend class U4CDCHitWire;

public:
  U4CDCHit(U4CDCCluster *cl = 0);
  U4CDCHit(const U4CDCHit &ht);
  virtual ~U4CDCHit();

  inline U4CDCCluster   *GetClusterPtr() const { return fClusterPtr; }
  inline U4CDCHitWire   *GetWirePtr   () const { return fWirePtr;    }

  virtual Int_t		GetLayerNumber() const;
  virtual Int_t		GetCellNumber () const;
  virtual Int_t		GetWireNumber () const;
  virtual Double_t 	Xwire     () const;
  virtual Double_t 	Ywire     () const;

  virtual Double_t	Xdrift () { if (!fDone) Update(); return fXdrift;  }
  virtual Double_t	DXdrift() { if (!fDone) Update(); return fDXdrift; }
  virtual Double_t      Vdrift () { if (!fDone) Update(); return fVdrift;  }
  virtual Double_t	Z ()	  { if (!fDone) Update(); return fZ	 ; }
  virtual Double_t	DZ()	  { if (!fDone) Update(); return fDZ	 ; }
  
  virtual Bool_t	IsSortable() const;
  virtual Int_t		Compare(const TObject *obj) const;

  virtual void 		DebugPrint(const Char_t *opt = "Brief");

protected:
  virtual void          Update();
private:
  virtual void          CalculatePosition();

protected:
  Bool_t	fDone;			// (1,0) = (cluster done, not yet)
  U4CDCCluster *fClusterPtr;		//! pointer to anode cluster
  Double_t	fXdrift;		// drift distance (cm)
  Double_t	fDXdrift;		// error on drift distance (cm)
  Double_t	fVdrift;		// error on drift distance (cm)
  Double_t	fZ;			// z from charge division  (cm)
  Double_t	fDZ;			// error on z              (cm)
  U4CDCHitWire *fWirePtr;		// pointer to wire this belongs to

  ClassDef(U4CDCHit, 1) 	// CDC Anode Hit
};

#endif
