//*************************************************************************
//* ---------------------
//* U4CDCHitLayer Class
//* ---------------------
//* (Description)
//*	Class to store "U4CDCHitCell"s for a single layer.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	U4CDCHitLayer		: Array containing "U4CDCHitCell"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "U4CDCHitLayer.h"

ClassImp(U4CDCHitLayer)

U4CDCHitLayer::U4CDCHitLayer() : TObjArray(200), fLayerNumber(0) 
{
}

U4CDCHitLayer::~U4CDCHitLayer()
{
	Delete();
}

Bool_t U4CDCHitLayer::IsSortable() const { return kTRUE; }

Int_t U4CDCHitLayer::Compare(const TObject *obj) const
{
	Int_t layer  = fLayerNumber;
	Int_t layero = ((U4CDCHitLayer *)obj)->fLayerNumber;
   	if (layer < layero) return -1;
   	else if (layer > layero) return 1;
   	else return 0;
}

Int_t U4CDCHitLayer::Append(U4CDCHitCell *cell)
{	
	Int_t layer = cell->GetLayerNumber();
	
	// Return value = -1:	new cell.
	//		=  0:	successfully appended.
	//		=  1:	new layer.
	
	if (GetEntries() == 0) {
		fLayerNumber     = layer;
		cell->fLayerPtr = this;
                Add(cell);
                return 0;
	} else {
		if (layer == fLayerNumber) {
			cell->fLayerPtr = this;
                        Add(cell);
                        return 0;
                } else {
                        return 1;
                }
        }
}

void U4CDCHitLayer::DebugPrint(const Char_t *opt)
{
	std::cerr 	<< "Layer = " << fLayerNumber << std::endl;
	TIter next(this);
	U4CDCHitCell *cell;
	while ((cell = (U4CDCHitCell *)next())) {
		cell->DebugPrint(opt);
	}
}

