//*************************************************************************
//* --------------------
//* U4CDCHitMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad hits.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2001/08/22  K.Hoshina	Original version.
//*************************************************************************

#include <iostream>
#include "U4CDCHitMakerBuf.h"
#include "U4CDCHitWire.h"

ClassImp(U4CDCHitMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
U4CDCHitMakerBuf::U4CDCHitMakerBuf(U4CDCTimeToXRelation *calib, const char *name,
	                       const char *title, U4CDCHitMaker *module,
	                       Int_t bufsize)
		 :U4VHitMakerBuf(name, title, module, bufsize), 
		  xWidth(100.), xPeak(10.), xSum(1000.),
	          fClusterData(0), fCalibPtr(calib)
{
}	       
	       
U4CDCHitMakerBuf::U4CDCHitMakerBuf(U4CDCHitMaker *module,
			      U4CDCTimeToXRelation *calib, 
			      const char *name,
	                      const char *title, 
	                      Int_t bufsize)
		 :U4VHitMakerBuf(name, title, module, bufsize), 
		  xWidth(100.), xPeak(10.), xSum(1000.),
	          fClusterData(0), fCalibPtr(calib)
{
}	       

//=====================================================================
//* destructor --------------------------------------------------------

U4CDCHitMakerBuf::~U4CDCHitMakerBuf()
{
}

//=====================================================================
//* SetCalibrationData ------------------------------------------------
void U4CDCHitMakerBuf::SetCalibrationData(U4CDCTimeToXRelation *calib)
{
	fDone  = kFALSE;
	fCalibPtr = calib;
}

//=====================================================================
//* SetClusterData     ------------------------------------------------
void U4CDCHitMakerBuf::SetClusterData(U4CDCClusterBuf *clbuf)
{
	fDone        = kFALSE;
	fInputDataBufPtr = clbuf;
	if (GetOutputBufPtr()->GetEntries() > 0) GetOutputBufPtr()->Delete();
}

//=====================================================================
//* DebugPrint --------------------------------------------------------
void U4CDCHitMakerBuf::DebugPrint(const Char_t *)
{
	TIter next(&fOutputBuf);
	U4CDCHitLayer *layerp;
	while ((layerp = (U4CDCHitLayer *)next())) {
		layerp->DebugPrint();
		std::cerr << "#Hit cells par layer = "
		     << layerp->GetEntries() << std::endl;
	}
}

//=====================================================================
//* MakeHits ----------------------------------------------------------

void U4CDCHitMakerBuf::MakeHits()
{
	if (!IsInitialized()) {
		std::cerr << "ERROR  U4CDCHitMakerBuf::MakeHits" << std::endl
		     << "...... cluster data not yet initialized." << std::endl
		     << "fInputDataBufPtr = " << fInputDataBufPtr << std::endl;
		return;
	} else if (fDone) {
		return;
	}
	
	fDone = kTRUE;
	
	U4CDCHitLayer *layerp = 0;
	U4CDCHitCell  *cellp  = 0;
	U4CDCHitWire  *wirep  = 0;
	
	TIter next(((U4CDCClusterBuf *)fInputDataBufPtr)->GetClusters());
	U4CDCCluster *clp;
	while ((clp = (U4CDCCluster *)next())) {
		Double_t w   = clp->GetWidth();
		Double_t pk  = clp->GetPeak();
		Double_t sum = clp->GetSum();
		if (w > xWidth && pk > xPeak && sum > xSum) {
			if (!layerp) {
				layerp = new U4CDCHitLayer();
				cellp  = new U4CDCHitCell();
				wirep  = new U4CDCHitWire(fCalibPtr);
			}
			U4CDCHit *ht = new U4CDCHit(clp);
			Int_t r1 = wirep->Append(ht);
			switch (r1) {
				case -1: {		// 1st hit
					cellp->Append(wirep);
					layerp->Append(cellp);
					fOutputBuf.Add(layerp);
					break;
				}
				case  0: break; 	// appended
				case  3: {		// new layer
					wirep = new U4CDCHitWire(fCalibPtr);
					wirep->Append(ht);
					cellp = new U4CDCHitCell();
					cellp->Append(wirep);
					layerp = new U4CDCHitLayer();
					layerp->Append(cellp);
					fOutputBuf.Add(layerp);
					break;
				}
				case  2: {		// new cell
					wirep = new U4CDCHitWire(fCalibPtr);
					wirep->Append(ht);
					cellp = new U4CDCHitCell();
					cellp->Append(wirep);
					layerp->Append(cellp);
					break;
				}
				case  1: {		// new wire
					wirep = new U4CDCHitWire(fCalibPtr);
					wirep->Append(ht);
					cellp->Append(wirep);
					break;
				}
			}
		}
	}
}

