#ifndef __U4CDCRAWGPIBDATUM__
#define __U4CDCRAWGPIBDATUM__

//*************************************************************************
//* --------------------
//* U4CDCRawGPIBDatum Class
//* --------------------
//* (Description)
//* 	Classes for a single GPIB datum.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCRawGPIBDatum
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*************************************************************************

#include "TObject.h"
#include "U4CDCUniversalRecordHeader.h"

// static variables are not declared as constant so as to allow
// different formats in future.

class U4CDCRawGPIBDatum : public TObject {
public:
  U4CDCRawGPIBDatum();
  U4CDCRawGPIBDatum(U4CDCRawGPIBDatum & rad);
  U4CDCRawGPIBDatum(ADC_t adcword[]);
  virtual ~U4CDCRawGPIBDatum();

  static void  SetFormat(U4CDCUniversalRecordHeader & urh);
  inline void  SetStationNo(Int_t n)    { fStationNo = n;    }
  inline void  SetChannelNo(Int_t n)    { fChannelNo = n;    }
  inline void  SetGPIB     (Double_t n) { fGPIB      = n;    }
  
  inline Int_t    GetStationNo() const  { return fStationNo; }
  inline Int_t    GetChannelNo() const  { return fChannelNo; }
  inline Double_t GetGPIB     () const  { return fGPIB;      }
  static Int_t    GetFormatID ()        { return gFormatID;  }
  static Int_t    GetRawLength()        { return gNswperdt;  }
  static Int_t    GetNchMax   ()        { return gNchMax;    }

  void Unpack(ADC_t adcword[]);
  void DebugPrint(const Char_t *opt = "");

private:
  Int_t	   fStationNo;			// Station number
  Int_t    fChannelNo;			// Channel number
  Double_t fGPIB;	        	// GPIB value
  static   Int_t gFormatID;		//! Format ID
  static   Int_t gNstabits;		//! No. of bits for station No
  static   Int_t gNchabits;		//! No. of bits for channel No
  static   Int_t gNswperdt;		//! No. of short words per channel
  static   Int_t gNchMax;		//! Maximum channel No

  ClassDef(U4CDCRawGPIBDatum, 1) // U4CDC raw data buffer
};

#endif
