//*************************************************************************
//* ===================
//*  U4VTXKalDetector Class
//* ===================
//*
//* (Description)
//* (Requires)
//* (Provides)
//* (Update Recored)
//*   2004/12/16  A.Yamaguchi       Original version.
//*
//*************************************************************************

#include <iostream>
#include <vector>
#include "JSFSteer.h"

#include "U4VTXKalDetector.h"
#include "U4VTXMeasLayer.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif
using namespace std;

ClassImp(U4VTXKalDetector)

//=====================================================================
//* constructor -------------------------------------------------------

U4VTXKalDetector::U4VTXKalDetector(Int_t n)
                : U4VKalDetector(n)
{
   SetOwner();
}

U4VTXKalDetector::U4VTXKalDetector(const TString &flag, Int_t n)
                : U4VKalDetector(n)
{
   if (flag != "FillLayer") {
      std::cerr << "<<< ERROR!! >>> flag should be \"FillLayer\"" << std::endl;
      abort();
   }
   Initialize();
   SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

U4VTXKalDetector::~U4VTXKalDetector()
{
   Clear();
}

//=====================================================================
//* Initialize --------------------------------------------------------

void U4VTXKalDetector::Initialize()
{
   Double_t a, z, dens, radl;
   TMaterial &air = *new TMaterial("VTXAir", "",
                                   a    = 14.00674*0.7+15.9994*0.3,
                                   z    = 7.3,
                                   dens = 1.205e-3,
                                   radl = 3.42e4, 0.);
   TMaterial &si  = *new TMaterial("VTXSi", "",
                                   a = 28.0855,
                                   z = 14.,
                                   dens = 2.33,
                                   radl = 9.36, 0.);

   Bool_t active = kTRUE;
   Bool_t dummy  = kFALSE;
   JSFEnv &env = *gJSF->Env();
   Int_t            nlayers  = env.GetValue("J4VTX.NLayers",4);
   vector<Double_t> radius   = env.GetDValue("J4VTX.Layer.Radius","2.4 3.6 4.8 6.0",nlayers);
   Double_t         sthick   = env.GetValue("J4VTX.Sensor.Thickness",0.033000);
   Double_t         slength  = env.GetValue("J4VTX.Sensor.Length",5.000);
   vector<Double_t> nsensors = env.GetDValue("J4VTX.NSensors","2 3 4 5",nlayers);  

   // create measurement layers of central tracker
   for (Int_t layer = 0; layer < nlayers; layer++) {
     Double_t zlen = slength * nsensors[layer];
     Add(new U4VTXMeasLayer(air, si, radius[layer], zlen, active));
     Add(new U4VTXMeasLayer(si, air, radius[layer] + sthick, zlen, dummy));
   }
}

//=====================================================================
//* GetMeasLayerPtr ---------------------------------------------------

const U4VTXMeasLayer *U4VTXKalDetector::GetMeasLayerPtr(Int_t index) const
{
   return dynamic_cast<U4VTXMeasLayer *>(At(index * 2));
}

